#!/usr/local/bin/python
# encoding: utf-8
"""
*Class to setup database object for the sherlock*

:Author:
    David Young

:Date Created:
    July 1, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import time
import MySQLdb as ms
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from dryxPython import mysql as dms
from fundamentals import tools, times


class database():

    """
    *The worker class for the database module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
    """
    # INITIALISATION

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new '_database' object")
        self.settings = settings
        return None

    def close(self):
        del self
        return None

    # METHOD ATTRIBUTES
    def get(self):
        """
        *get the database object*

        **Return:**
            - ``self.transientsDbConn, self.cataloguesDbConn`` -- two database connections
        """
        self.log.debug('starting the ``get`` method')
        self._setup_database_connections()
        self.log.debug('completed the ``get`` method')
        return self.transientsDbConn, self.cataloguesDbConn, self.pmDbConn

    def _setup_database_connections(
            self):
        """
        *setup database connections for transient and catalogue databases*
        """
        self.log.debug('starting the ``_setup_database_connections`` method')

        from subprocess import Popen, PIPE, STDOUT

        # SETUP TUNNEL IF REQUIRED
        if self.settings["ssh tunnel"]["use tunnel"] is True:
            # TEST TUNNEL DOES NOT ALREADY EXIST
            sshPort = self.settings["ssh tunnel"]["port"]
            connected = self._checkServer(
                self.settings["database settings"]["static catalogues"]["host"], sshPort)
            if connected:
                self.log.debug('ssh tunnel already exists - moving on')
            else:
                # GRAB TUNNEL SETTINGS FROM SETTINGS FILE
                ru = self.settings["ssh tunnel"]["remote user"]
                rip = self.settings["ssh tunnel"]["remote ip"]
                rh = self.settings["ssh tunnel"]["remote datbase host"]

                cmd = "ssh -fnN %(ru)s@%(rip)s -L %(sshPort)s:%(rh)s:3306" % locals()
                p = Popen(cmd, shell=True, close_fds=True)
                output = p.communicate()[0]
                self.log.debug('output: %(output)s' % locals())

                # TEST CONNECTION - QUIT AFTER SO MANY TRIES
                connected = False
                count = 0
                while not connected:
                    connected = self._checkServer(
                        self.settings["database settings"]["static catalogues"]["host"], sshPort)
                    time.sleep(1)
                    count += 1
                    if count == 5:
                        self.log.error(
                            'cound not setup tunnel to remote datbase' % locals())
                        sys.exit(0)
        else:
            sshPort = 3306

        # SETUP A DATABASE CONNECTION FOR THE STATIC CATALOGUES
        host = self.settings["database settings"]["static catalogues"]["host"]
        user = self.settings["database settings"]["static catalogues"]["user"]
        passwd = self.settings["database settings"][
            "static catalogues"]["password"]
        dbName = self.settings["database settings"]["static catalogues"]["db"]
        if self.settings["database settings"]["static catalogues"]["sshPort"]:
            thisPort = sshPort
        else:
            thisPort = False
        thisConn = ms.connect(
            host=host,
            user=user,
            passwd=passwd,
            db=dbName,
            port=thisPort,
            use_unicode=True,
            charset='utf8'
        )
        thisConn.autocommit(True)
        self.log.debug('cataloguesDbConn: %s' % (thisConn,))
        self.cataloguesDbConn = thisConn

        # SETUP DATABASE CONNECTION FOR TRANSIENTS DATABASE
        host = self.settings["database settings"]["transients"]["host"]
        user = self.settings["database settings"]["transients"]["user"]
        passwd = self.settings["database settings"][
            "transients"]["password"]
        dbName = self.settings["database settings"]["transients"]["db"]
        if self.settings["database settings"]["transients"]["sshPort"]:
            thisPort = sshPort
        else:
            thisPort = False
        thisConn = ms.connect(
            host=host,
            user=user,
            passwd=passwd,
            db=dbName,
            port=thisPort,
            use_unicode=True,
            charset='utf8'
        )
        thisConn.autocommit(True)
        self.log.debug('transientsDbConn: %s' % (thisConn,))
        self.transientsDbConn = thisConn

        # SETUP DATABASE CONNECTION FOR PESSTO MARSHALL DATABASE
        if "pessto marshall" in self.settings["database settings"]:
            host = self.settings["database settings"][
                "pessto marshall"]["host"]
            user = self.settings["database settings"][
                "pessto marshall"]["user"]
            passwd = self.settings["database settings"][
                "pessto marshall"]["password"]
            dbName = self.settings["database settings"][
                "pessto marshall"]["db"]
            if self.settings["database settings"]["pessto marshall"]["sshPort"]:
                thisPort = sshPort
            else:
                thisPort = False
            thisConn = ms.connect(
                host=host,
                user=user,
                passwd=passwd,
                db=dbName,
                port=thisPort,
                use_unicode=True,
                charset='utf8'
            )
            thisConn.autocommit(True)
            self.log.debug('pmDbConn: %s' % (thisConn,))
            self.pmDbConn = thisConn
        else:
            self.pmDbConn = False

        self.log.debug('completed the ``_setup_database_connections`` method')
        return None

    def _checkServer(self, address, port):
        """
        *Check that the TCP Port we've decided to use for tunnelling is available*
        """
        self.log.debug('starting the ``_checkServer`` method')

        # CREATE A TCP SOCKET
        import socket
        s = socket.socket()
        self.log.debug(
            """Attempting to connect to `%(address)s` on port `%(port)s`""" % locals())
        try:
            s.connect((address, port))
            self.log.debug(
                """Connected to `%(address)s` on port `%(port)s`""" % locals())
            return True
        except socket.error, e:
            self.log.warning(
                """Connection to `%(address)s` on port `%(port)s` failed - try again: %(e)s""" % locals())
            return False

        return None

    # xt-class-method


if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*The base importer for sherlock imports*

:Author:
    David Young

:Date Created:
    August 25, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import codecs
import string

import re
from docopt import docopt
from dryxPython import mysql as dms
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
# from ..__init__ import *


class _base_importer():

    """
    *The worker class for the _base_importer module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``pathToDataFIle`` -- path to the _base_importer data file
        - ``version`` -- version of the _base_importer catalogue
        - ``catalogueName`` -- name of the catalogue
        - ``coordinateList`` -- list of coordinates (needed for some streamed tables)
        - ``cataloguesDbConn`` -- catalogues database connection (default false - will be setup here if false)


    .. todo::

        - @review: when complete, clean _base_importer class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
            self,
            log,
            settings=False,
            pathToDataFile=False,
            version=False,
            catalogueName="",
            coordinateList=[],
            cataloguesDbConn=False
    ):
        self.log = log
        log.debug("instansiating a new '_base_importer' object")
        self.settings = settings
        self.pathToDataFile = pathToDataFile
        self.version = version
        self.catalogueName = catalogueName
        self.coordinateList = coordinateList
        self.cataloguesDbConn = cataloguesDbConn

        # xt-self-arg-tmpx

        # INITIAL ACTIONS
        # SETUP DATABASE CONNECTIONS
        if cataloguesDbConn == False:
            from sherlock import database
            db = database(
                log=self.log,
                settings=self.settings
            )
            self.transientsDbConn, self.cataloguesDbConn, self.pmDbConn = db.get()

        if pathToDataFile:
            pathToReadFile = pathToDataFile
            try:
                self.log.debug("attempting to open the file %s" %
                               (pathToReadFile,))
                readFile = codecs.open(pathToReadFile, mode='r')
                self.catData = readFile.read()
                readFile.close()
            except IOError, e:
                message = 'could not open the file %s' % (pathToReadFile,)
                self.log.critical(message)
                raise IOError(message)
            readFile.close()
        else:
            self.catData = None

        # BUILD VERSION TEXT
        if self.version:
            self.version = "_v" + \
                self.version.replace(" ", "").replace(
                    "v", "").replace(".", "_")
        else:
            self.version = ""
        version = self.version

        # BUILD THE DATABASE TABLE NAME
        self.dbTableName = "tcs_cat_%(catalogueName)s%(version)s" % locals()
        self.primaryIdColumnName = "primaryId"
        self.databaseInsertbatchSize = 2500
        self.raColName = "raDeg"
        self.declColName = "decDeg"
        self.uniqueKeyList = [self.raColName, "decDeg"]

        return None

    # Method Attributes
    def get(self):
        """
        *get the _base_importer object*

        **Return:**
            - ``_base_importer``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        # self.dictList = self.create_dictionary_of__base_importer()
        # self.add_data_to_database_table()
        self.add_htmids_to_database_table()

        self.log.info('completed the ``get`` method')
        return _base_importer

    def create_dictionary_of__base_importer(
            self):
        """
        *create dictionary of _base_importer*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean create_dictionary_of__base_importer method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``create_dictionary_of__base_importer`` method')

        dictList = []
        lines = string.split(self.catData, '\n')
        inserts = [
            11, 25, 51, 57, 64, 71, 75, 78, 81, 89, 97, 106, 110, 134, 158, 181]
        keys = ["raDeg", "decDeg", "name", "descrip", "rmag", "bmag", "comment", "r_psf_class", "b_psf_class", "z",
                "src_cat_name", "src_cat_z", "qso_prob", "x_name", "r_name", "alt_id1", "alt_id2"]

        totalCount = len(lines)
        count = 0

        for line in lines:
            count += 1
            if count > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            if count > totalCount:
                count = totalCount
            print "%(count)s / %(totalCount)s _base_importer data added to memory" % locals()

            thisDict = {}
            for insert in inserts:
                line.replace("–", "-")
                line = line[:insert] + "|" + line[insert:]

            theseValues = string.split(line, '|')
            for k, v in zip(keys, theseValues):
                v = v.strip()
                if len(v) == 0 or v == "-":
                    v = None
                thisDict[k] = v
            dictList.append(thisDict)

        self.log.info(
            'completed the ``create_dictionary_of__base_importer`` method')
        return dictList

    # use the tab-trigger below for new method
    def add_data_to_database_table(
            self):
        """
        *add data to database table*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean add_data_to_database_table method
            - @review: when complete add logging
        """
        self.log.info('starting the ``add_data_to_database_table`` method')

        dms.insert_list_of_dictionaries_into_database(
            dbConn=self.cataloguesDbConn,
            log=self.log,
            dictList=self.dictList,
            dbTableName=self.dbTableName,
            uniqueKeyList=self.uniqueKeyList,
            batchSize=self.databaseInsertbatchSize
        )

        self.log.info('completed the ``add_data_to_database_table`` method')
        return None

    # use the tab-trigger below for new method
    def add_htmids_to_database_table(
            self):
        """
        *add htmids to database table*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean add_htmids_to_database_table method
            - @review: when complete add logging
        """
        self.log.info('starting the ``add_htmids_to_database_table`` method')

        tableName = self.dbTableName
        print "Adding HTMIds to %(tableName)s" % locals()

        from dryxPython import mysql as dms
        dms.add_HTMIds_to_mysql_tables.add_HTMIds_to_mysql_tables(
            raColName=self.raColName,
            declColName=self.declColName,
            tableName=self.dbTableName,
            dbConn=self.cataloguesDbConn,
            log=self.log,
            primaryIdColumnName=self.primaryIdColumnName
        )

        self.log.info('completed the ``add_htmids_to_database_table`` method')
        return None

    def _update_database_helper_table(
            self):
        """
        *update dataasbe helper table*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_database_helper_table method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_update_database_helper_table`` method')

        tableName = self.dbTableName

        sqlQuery = u"""
            update tcs_helper_catalogue_tables_info set last_updated = now() where table_name = "%(tableName)s";
        """ % locals()

        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info('completed the ``_update_database_helper_table`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method



import os
import nose
import shutil
import yaml
from .. import tmp_sdss
from sherlock.utKit import utKit

# load settings
stream = file(
    "/Users/Dave/git_repos/sherlock/example_settings.yaml", 'r')
settings = yaml.load(stream)
stream.close()


# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()

# xnose-class-to-test-main-command-line-function-of-module


class test_tmp_sdss():

    def test_tmp_sdss_function(self):
        kwargs = {}
        kwargs["log"] = log
        kwargs["settings"] = settings
        kwargs["pathToDataFile"] = "/Users/Dave/Desktop/milliquas.txt"
        kwargs["catalogueName"] = "tcs_cat_sdss_dr12_spect_galaxies_qsos"
        # xt-kwarg_key_and_value

        testObject = tmp_sdss(**kwargs)
        testObject.get()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Build and import NED-D table in datasbe*

:Author:
    David Young

:Date Created:
    September 7, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import csv
import time
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from dryxPython import mysql as dms
from dryxPython import astrotools as dat
from neddy import namesearch
from fundamentals import tools, times
from .._base_importer import _base_importer
from . import *


class ned_conesearch(_base_importer):

    """
    *The worker class for the ned_conesearch importer module*
    """

    def get(self):
        """
        *import the NED-D database into the database and do other cleanup tricks.*
        """
        self.log.info('starting the ``get`` method')

        self.dbTableName = "tcs_cat_ned_stream"

        self._get_ned_names()
        self._update_ned_query_history()
        self.databaseInsertbatchSize = 10000
        self.get_metadata_for_galaxies()
        self.add_htmids_to_database_table()

        self.log.info('completed the ``get`` method')
        return ned_conesearch

    def get_metadata_for_galaxies(
            self):
        """
        *get metadata for galaxies*
        """
        self.log.info('starting the ``get_metadata_for_galaxies`` method')

        self.batchSize = 5000.

        total, batches = self._count_galaxies_requiring_metadata()
        print "%(total)s NED sources require metadata. Need to send %(batches)s batch requests to NED." % locals()

        totalBatches = self.batches
        thisCount = 0

        # FOR EACH BATCH, GET THE GALAXY IDs, QUERY NED AND UPDATE THE DATABASE
        while self.total:
            thisCount += 1
            self._get_single_batch_of_galaxies_needing_metadata()
            self._query_ned_and_add_results_to_database(thisCount)
            total, batches = self._count_galaxies_requiring_metadata()
            print "%(total)s NED sources still require metadata." % locals()

        self.log.info('completed the ``get_metadata_for_galaxies`` method')
        return None

    def _count_galaxies_requiring_metadata(
            self):
        """
        *count galaxies requiring metadata*

        **Return:**
            - ``self.total``, ``self.batches`` -- total number of galaxies needing metadata & the number of batches required to be sent to NED
        """
        self.log.info(
            'starting the ``_count_galaxies_requiring_metadata`` method')

        tableName = self.dbTableName

        sqlQuery = u"""
            select count(*) as count from %(tableName)s where raDeg is null and download_error != 1
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        self.total = rows[0]["count"]
        self.batches = int(self.total / self.batchSize) + 1

        if self.total == 0:
            self.batches = 0

        self.log.info(
            'completed the ``_count_galaxies_requiring_metadata`` method')
        return self.total, self.batches

    def _get_single_batch_of_galaxies_needing_metadata(
            self):
        """ get batch of galaxies needing metadata

        **Return:**
            - ``len(self.theseIds)`` -- the number of NED IDs returned
        """
        self.log.info(
            'starting the ``_get_single_batch_of_galaxies_needing_metadata`` method')

        bs = int(self.batchSize)
        tableName = self.dbTableName

        print "Requesting the next %(bs)s sources requiring metadata from the %(tableName)s table" % locals()

        # SELECT THE DATA FROM NED TABLE
        self.theseIds = []

        sqlQuery = u"""
            select ned_name from %(tableName)s where raDeg is null and download_error != 1 limit %(bs)s;
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        for row in rows:
            self.theseIds.append(row["ned_name"])

        self.log.info(
            'completed the ``_get_single_batch_of_galaxies_needing_metadata`` method')

        return len(self.theseIds)

    def _query_ned_and_add_results_to_database(
            self,
            batchCount):
        """
        *query ned and add results to database*

        **Key Arguments:**
            - ``batchCount`` - the index number of the batch sent to NED
        """
        self.log.info(
            'starting the ``_query_ned_and_add_results_to_database`` method')

        from datetime import datetime, date, time
        tableName = self.dbTableName

        # QUERY NED WITH BATCH
        totalCount = len(self.theseIds)
        print "requesting metadata from NED for %(totalCount)s galaxies (batch %(batchCount)s)" % locals()
        search = namesearch(
            log=self.log,
            names=self.theseIds,
            quiet=True
        )
        results = search.get()
        numResults = len(results)
        print "%(numResults)s results returned from ned -- starting to add to database ..." % locals()

        # CLEAN THE RETURNED DATA AND UPDATE DATABASE
        totalCount = len(results)
        count = 0
        sqlQuery = ""
        for thisDict in results:
            thisDict["tableName"] = tableName
            count += 1
            for k, v in thisDict.iteritems():
                if not v or len(v) == 0:
                    thisDict[k] = "null"
                if k in ["major_diameter_arcmin", "minor_diameter_arcmin"] and (":" in v or "?" in v or "<" in v):
                    thisDict[k] = v.replace(":", "").replace(
                        "?", "").replace("<", "")
                if isinstance(v, str) and '"' in v:
                    thisDict[k] = v.replace('"', '\\"')
            if "Input name not" not in thisDict["input_note"] and "Same object as" not in thisDict["input_note"]:
                try:
                    thisDict[
                        "raDeg"] = dat.ra_sexegesimal_to_decimal.ra_sexegesimal_to_decimal(thisDict["ra"])
                    thisDict[
                        "decDeg"] = dat.declination_sexegesimal_to_decimal.declination_sexegesimal_to_decimal(thisDict["dec"])
                except:
                    continue

                if thisDict["major_diameter_arcmin"] != "null":
                    try:
                        float(thisDict["major_diameter_arcmin"])
                    except:
                        print "major_diameter_arcmin is " + thisDict["major_diameter_arcmin"] + ". Changing to null."
                        thisDict["major_diameter_arcmin"] = "null"

                if thisDict["major_diameter_arcmin"] != "null":
                    try:
                        float(thisDict["major_diameter_arcmin"])
                    except:
                        print "major_diameter_arcmin is " + thisDict["major_diameter_arcmin"] + ". Changing to null."
                        thisDict["major_diameter_arcmin"] = "null"

                if thisDict["major_diameter_arcmin"] != "null":
                    try:
                        float(thisDict["major_diameter_arcmin"])
                    except:
                        print "major_diameter_arcmin is " + thisDict["major_diameter_arcmin"] + ". Changing to null."
                        thisDict["major_diameter_arcmin"] = "null"

                sqlQuery += u"""
                    update %(tableName)s
                        set redshift_quality = "%(redshift_quality)s",
                        redshift = %(redshift)s,
                        hierarchy = "%(hierarchy)s",
                        object_type = "%(object_type)s",
                        major_diameter_arcmin = %(major_diameter_arcmin)s,
                        morphology = "%(morphology)s",
                        magnitude_filter = "%(magnitude_filter)s",
                        ned_notes = "%(ned_notes)s",
                        eb_v = %(eb-v)s,
                        raDeg = %(raDeg)s,
                        radio_morphology = "%(radio_morphology)s",
                        activity_type = "%(activity_type)s",
                        minor_diameter_arcmin = %(minor_diameter_arcmin)s,
                        decDeg = %(decDeg)s,
                        redshift_err = %(redshift_err)s
                    where ned_name = "%(input_name)s";\n
                """ % thisDict
        sqlQuery = sqlQuery.replace('"null"', 'null')

        if len(sqlQuery) != 0:

            startTime = dcu.get_now_sql_datetime()
            print "Executing the sql query ..."
            rows = dms.execute_mysql_read_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )
            endTime = dcu.get_now_sql_datetime()
            runningTime = dcu.calculate_time_difference(startTime, endTime)
            percent = (float(count) / float(totalCount)) * 100.
            # sys.stdout.write("\x1b[1A")
            print "%(count)s / %(totalCount)s (%(percent)1.1f%%) message" % locals()
            print "... and finished (time taken: %(runningTime)s) " % locals()
        else:
            for thisId in self.theseIds:
                sqlQuery = u"""
                    update %(tableName)s set download_error = 1 where ned_name = "%(thisId)s"
                """ % locals()
                dms.execute_mysql_write_query(
                    sqlQuery=sqlQuery,
                    dbConn=self.cataloguesDbConn,
                    log=self.log
                )

        print "%(count)s/%(totalCount)s galaxy metadata batch entries added to database" % locals()
        if count < totalCount:
            # Cursor up one line and clear line
            sys.stdout.write("\x1b[1A\x1b[2K")

        sqlQuery = u"""
            update tcs_helper_catalogue_tables_info set last_updated = now() where table_name = "%(tableName)s"
        """ % locals()
        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info(
            'completed the ``_query_ned_and_add_results_to_database`` method')
        return None

    def _get_ned_names(
            self):
        """
        *get ned names*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _get_ned_names method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_get_ned_names`` method')

        from neddy import conesearch

        tableName = self.dbTableName

        numSearches = len(self.coordinateList)
        if numSearches:
            print "Requesting the names of the NED sources found within %(numSearches)s conesearch areas" % locals()

        names, searchParams = conesearch(
            log=self.log,
            radiusArcsec=self.settings["ned stream search radius arcec"],
            nearestOnly=False,
            unclassified=True,
            quiet=False,
            listOfCoordinates=self.coordinateList,
            outputFilePath=False,
            verbose=False
        ).get_crossmatch_names()

        manyValueList = []
        for n in names:
            manyValueList.append((n,))

        numResults = len(manyValueList)
        if numSearches:
            print "Inserting the names of the %(numResults)s matched NED sources into `tcs_cat_ned_stream` table" % locals()
            print ""

        sqlQuery = u"""
            INSERT ignore into tcs_cat_ned_stream (ned_name, dateCreated) values (%s, now())
        """
        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log,
            manyValueList=manyValueList)

        self.log.info('completed the ``_get_ned_names`` method')
        return None

    # use the tab-trigger below for new method
    def _update_ned_query_history(
            self):
        """
        *update ned query history*

        **Key Arguments:**
            - ``coordinateList`` - the coordinates that where queried

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_ned_query_history method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_update_ned_query_history`` method')

        from dryxPython import mysql as dms

        manyValueList = []
        radius = self.settings["ned stream search radius arcec"]
        from datetime import datetime, date, time
        now = datetime.now()
        now = now.strftime("%Y-%m-%d %H:%M:%S")

        for i, coord in enumerate(self.coordinateList):
            coord = coord.split(" ")
            manyValueList.append((coord[0], coord[1], radius, now))

        num = len(self.coordinateList)
        if num:
            print "Updating the `tcs_helper_ned_query_history` for the %(num)s new initial conesearches just performed" % locals()

        sqlQuery = u"""
            INSERT into tcs_helper_ned_query_history (raDeg, decDeg, arcsecRadius, dateQueried) values (%s, %s, %s, %s)
        """
        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log,
            manyValueList=manyValueList
        )

        dms.add_HTMIds_to_mysql_tables.add_HTMIds_to_mysql_tables(
            raColName="raDeg",
            declColName="decDeg",
            tableName="tcs_helper_ned_query_history",
            dbConn=self.cataloguesDbConn,
            log=self.log,
            primaryIdColumnName="primaryId"
        )

        self.log.info('completed the ``_update_ned_query_history`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method



#!/usr/local/bin/python
# encoding: utf-8
"""
*Import PESSTO Marshall tables into crossmatch catalogues*

:Author:
    David Young

:Date Created:
    August 25, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython

# xdocopt-usage-tempx
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import codecs
import string
import yaml

import re
from docopt import docopt
from dryxPython import mysql as dms
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from ._base_importer import _base_importer


class pessto_marshall_streams(_base_importer):

    """
    *The worker class for the pessto_marshall_streams module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``pathToDataFIle`` -- path to the pessto_marshall_streams data file
        - ``version`` -- version of the pessto_marshall_streams catalogue


    .. todo::

        - @review: when complete, clean pessto_marshall_streams class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the pessto_marshall_streams object*

        **Return:**
            - ``pessto_marshall_streams``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        yamlFilePath = '/'.join(string.split(__file__, '/')
                                [:-1]) + "/resources/pessto_marshall_table_selects.yaml"

        stream = file(yamlFilePath, 'r')
        pesstoQueries = yaml.load(stream)
        stream.close()

        self.primaryIdColumnName = "primaryId"
        self.raColName = "raDeg"
        self.declColName = "decDeg"

        for k, v in pesstoQueries["pessto queries"].iteritems():
            self.dbTableName = k
            self.databaseInsertbatchSize = 500
            self.marshallQuery = v
            self.dictList = self.create_dictionary_of_pessto_marshall_streams()
            self.add_data_to_database_table()
            self.add_htmids_to_database_table()
            self._update_database_helper_table()

        self.log.info('completed the ``get`` method')
        return pessto_marshall_streams

    def create_dictionary_of_pessto_marshall_streams(
            self):
        """
        *create dictionary of pessto_marshall_streams*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean create_dictionary_of_pessto_marshall_streams method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``create_dictionary_of_pessto_marshall_streams`` method')

        dictList = []
        tableName = self.dbTableName

        rows = dms.execute_mysql_read_query(
            sqlQuery=self.marshallQuery,
            dbConn=self.pmDbConn,
            log=self.log
        )

        totalCount = len(rows)
        count = 0

        for row in rows:
            count += 1
            if count > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            print "%(count)s / %(totalCount)s `%(tableName)s` data added to memory" % locals()
            dictList.append(dict(row))

        self.log.info(
            'completed the ``create_dictionary_of_pessto_marshall_streams`` method')
        return dictList

    # use the tab-trigger below for new method
    def _update_database_helper_table(
            self):
        """
        *update dataasbe helper table*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_database_helper_table method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_update_database_helper_table`` method')

        tableName = self.dbTableName

        sqlQuery = u"""
            update tcs_helper_catalogue_tables_info set last_updated = now() where table_name = "%(tableName)s";
        """ % locals()

        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info('completed the ``_update_database_helper_table`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx

# xt-class-tmpx


###################################################################
# PUBLIC FUNCTIONS                                                #
###################################################################
# xt-worker-def

# use the tab-trigger below for new function
# xt-def-with-logger

###################################################################
# PRIVATE (HELPER) FUNCTIONS                                      #
###################################################################

if __name__ == '__main__':
    main()



import os
import nose
import shutil
import yaml
from sherlock import conesearcher
from sherlock import database
from sherlock.utKit import utKit

# load settings
stream = file(
    "/Users/Dave/git_repos/sherlock/example_settings.yaml", 'r')
settings = yaml.load(stream)
stream.close()


# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()


# xnose-class-to-test-main-command-line-function-of-module
class test_conesearcher():

    def test_conesearcher_function(self):
        kwargs = {}
        kwargs["log"] = log
        kwargs["settings"] = settings
        # xt-kwarg_key_and_value
        testObject = database(**kwargs)
        transientsDbConn, cataloguesDbConn = testObject.get()

        kwargs["ra"] = 56.23423672301499
        kwargs["dec"] = 0.2221599106133527
        kwargs["tableName"] = "tcs_cat_v_sdss_dr9_spect_qsos"
        kwargs["radius"] = "40.0"
        kwargs["dbConn"] = cataloguesDbConn
        kwargs["settings"] = settings
        kwargs["htmLevel"] = 16
        kwargs["queryType"] = 1
        testObject = conesearcher(**kwargs)
        testObject.get()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Import IFS galaxies from the IFS galaxy stream*

:Author:
    David Young

:Date Created:
    October  8, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import codecs
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from dryxPython import webcrawlers as dwc
from dryxPython import astrotools as dat
from ._base_importer import _base_importer


class ifs_galaxies(_base_importer):
    """
    *The worker class for the ifs_galaxies module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``pathToDataFIle`` -- path to the pessto_marshall_streams data file
        - ``version`` -- version of the pessto_marshall_streams catalogue

    .. todo::

        - @review: when complete, clean ifs_galaxies class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the ifs_galaxies object*

        **Return:**
            - ``ifs_galaxies``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        self.primaryIdColumnName = "primaryId"
        self.raColName = "raDeg"
        self.declColName = "decDeg"

        self.dbTableName = "tcs_cat_ifs_stream"
        self.databaseInsertbatchSize = 500
        self.dictList = self._download_and_parse_ifs_galaxy_csv()
        self.add_data_to_database_table()
        self.add_htmids_to_database_table()
        self._update_database_helper_table()

        self.log.info('completed the ``get`` method')
        return ifs_galaxies

    def _download_and_parse_ifs_galaxy_csv(
            self):
        """
        *download and parse ifs galaxy csv*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _download_and_parse_ifs_galaxy_csv method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``_download_and_parse_ifs_galaxy_csv`` method')

        urlDoc = dwc.singleWebDocumentDownloader(
            url=self.settings["ifs galaxies url"],
            downloadDirectory="/tmp",
            log=self.log,
            timeStamp=1,
            credentials=False
        )

        pathToReadFile = urlDoc
        try:
            self.log.debug("attempting to open the file %s" %
                           (pathToReadFile,))
            readFile = codecs.open(pathToReadFile, encoding='utf-8', mode='r')
            thisData = readFile.readlines()
            readFile.close()
        except IOError, e:
            message = 'could not open the file %s' % (pathToReadFile,)
            self.log.critical(message)
            raise IOError(message)

        dictList = []
        columns = ["name", "raDeg", "decDeg", "z"]
        for line in thisData:
            thisDict = {}
            line = line.strip()
            line = line.replace("\t", " ")
            values = line.split("|")
            if len(values) > 3:
                thisDict["name"] = values[0].strip()

                try:
                    raDeg = dat.ra_sexegesimal_to_decimal.ra_sexegesimal_to_decimal(values[
                                                                                    1].strip())
                except:
                    raDeg = float(values[1].strip())
                thisDict["raDeg"] = raDeg
                try:
                    decDeg = dat.declination_sexegesimal_to_decimal.declination_sexegesimal_to_decimal(values[
                                                                                                       2].strip())
                except:
                    decDeg = float(values[2].strip())
                thisDict["decDeg"] = decDeg
                try:
                    z = float(values[3].strip())
                    if z > 0.:
                        thisDict["z"] = float(values[3].strip())
                    else:
                        thisDict["z"] = None
                except:
                    thisDict["z"] = None
                dictList.append(thisDict)

        self.log.info(
            'completed the ``_download_and_parse_ifs_galaxy_csv`` method')
        return dictList

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx



import utKit
import cl_utils
from classifier import classifier
from _database import database
from _crossmatcher import crossmatcher
from _conesearcher import conesearcher
from cleanup_database_tables import cleanup_database_tables



#!/usr/local/bin/python
# encoding: utf-8
"""
*CL Utils for the Sherlock importers*

:Author:
    David Young

:Date Created:
    August 25, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython

Usage:
    sherlock-importers cat <cat_name> <pathToDataFile> <cat_version> [-s <pathToSettingsFile>]
    sherlock-importers stream <stream_name> [-s <pathToSettingsFile>]

    -h, --help            show this help message
    -v, --version         show version
    -s, --settings        the settings file
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from sherlock.imports import milliquas as milliquasImporter
from sherlock.imports import veron as veronImporter
from sherlock.imports import tmp_sdss as sdssImporter
from sherlock.imports import pessto_marshall_streams as pesstoImporter
from sherlock.imports import ifs_galaxies as ifsImporter
from sherlock.imports.ned_d import ned_d as nedImporter


def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]


def main(arguments=None):
    """
    *The main function used when ``importers_clutils.py`` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="WARNING",
        options_first=False,
        projectName="sherlock"
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # unpack remaining cl arguments using `exec` to setup the variable names
    # automatically
    for arg, val in arguments.iteritems():
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        if isinstance(val, str) or isinstance(val, unicode):
            exec(varname + " = '%s'" % (val,))
        else:
            exec(varname + " = %s" % (val,))
        if arg == "--dbConn":
            dbConn = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE importers_clutils.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in locals() and interactiveFlag:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    # call the worker function
    # x-if-settings-or-database-credientials
    if cat:
        if cat_name == "milliquas":
            testObject = milliquasImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()
        if cat_name == "veron":
            testObject = veronImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()
        if "sdss" in cat_name:
            testObject = sdssImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()
        if "ned_d" in cat_name:
            testObject = nedImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()

    elif stream:
        if "pessto" in stream_name:
            testObject = pesstoImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()
        if "ifs" in stream_name:
            testObject = ifsImporter(
                log=log,
                settings=settings,
                pathToDataFile=pathToDataFile,
                version=cat_version,
                catalogueName=cat_name
            )
            testObject.get()

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE importers_clutils.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return


###################################################################
# CLASSES                                                         #
###################################################################
# xt-class-module-worker-tmpx
# xt-class-tmpx


###################################################################
# PUBLIC FUNCTIONS                                                #
###################################################################
# xt-worker-def

# use the tab-trigger below for new function
# xt-def-with-logger

###################################################################
# PRIVATE (HELPER) FUNCTIONS                                      #
###################################################################

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*Import tmp_sdss catagloue from plain text file*

:Author:
    David Young

:Date Created:
    August 25, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython

# xdocopt-usage-tempx
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import codecs
import string

import re
from docopt import docopt
from dryxPython import mysql as dms
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from ._base_importer import _base_importer


class tmp_sdss(_base_importer):

    """
    *The worker class for the tmp_sdss module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``pathToDataFIle`` -- path to the tmp_sdss data file
        - ``version`` -- version of the tmp_sdss catalogue


    .. todo::

        - @review: when complete, clean tmp_sdss class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the tmp_sdss object*

        **Return:**
            - ``tmp_sdss``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        self.dbTableName = self.catalogueName
        self.primaryIdColumnName = "qubPrimaryId"
        self.raColName = "ra"
        self.declColName = "dec_"

        # self.dictList = self.create_dictionary_of_tmp_sdss()
        # self.add_data_to_database_table()
        self.create_master_id()
        self.set_master_obj_types()

        # self.add_htmids_to_database_table()

        self.log.info('completed the ``get`` method')
        return tmp_sdss

    def create_master_id(
            self):
        """
        *create master id*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean create_master_id method
            - @review: when complete add logging
        """
        self.log.info('starting the ``create_master_id`` method')

        batchSize = 1250
        t = self.dbTableName

        if "photo" in t:
            totalRows = 500000000
        else:
            sqlQuery = u"""
                select distinct objId from %(t)s where qubMasterFlag = 2
            """ % locals()
            count = dms.execute_mysql_read_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )
            totalRows = len(count)
        count = ""

        total = totalRows
        batches = int(total / batchSize)

        start = 0
        end = 0
        theseBatches = []
        for i in range(batches):
            end = end + batchSize
            if end > total:
                end = total
            start = i * batchSize

            if start > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            percent = (float(end) / float(totalRows)) * 100.
            print "%(end)s / %(totalRows)s (%(percent)1.1f%%) masterFlags updated in %(t)s" % locals()

            sqlQuery = u"""
                select distinct objid from %(t)s where qubMasterFlag = 2 limit 0, %(batchSize)s  
            """ % locals()

            rows = dms.execute_mysql_read_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )
            sqlQuery = ""
            for row in rows:
                objid = row["objid"]
                sqlQuery = sqlQuery + \
                    u"""\nupdate %(t)s set qubMasterFlag = 1 where qubPrimaryId = (select * from (SELECT qubPrimaryId FROM %(t)s where objId = %(objid)s  order by clean desc limit 1) as alias);
update %(t)s set qubMasterFlag = 0 where objId = %(objid)s  and qubMasterFlag != 1;""" % locals()

            dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log,
                Force=False
            )

        self.log.info('completed the ``create_master_id`` method')
        return None

    # use the tab-trigger below for new method
    def set_master_obj_types(
            self):
        """
        *set master obj types*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean set_master_obj_types method
            - @review: when complete add logging
        """
        self.log.info('starting the ``set_master_obj_types`` method')

        batchSize = 1250
        t = self.dbTableName

        if "photo" in t:
            totalRows = 500000000
            return
        else:
            sqlQuery = u"""
                select distinct objid from %(t)s  where objType = "Q"
            """ % locals()
            count = dms.execute_mysql_read_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )
            totalRows = len(count)
        count = ""

        total = totalRows
        batches = int(total / batchSize)

        start = 0
        end = 0
        theseBatches = []
        for i in range(batches):
            end = end + batchSize
            if end > total:
                end = total
            start = i * batchSize

            if start > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            percent = (float(end) / float(totalRows)) * 100.
            print "%(end)s / %(totalRows)s (%(percent)1.1f%%) master objIds updated in %(t)s" % locals()

            sqlQuery = u"""
                select distinct objid from %(t)s  where objType = "Q" limit %(start)s , %(batchSize)s  
            """ % locals()

            rows = dms.execute_mysql_read_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )
            sqlQuery = ""
            for row in rows:
                objid = row["objid"]
                sqlQuery = sqlQuery + \
                    u"""\nupdate %(t)s set objType = "Q" where objId = %(objid)s;""" % locals(
                    )

            dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log,
                Force=False
            )

        self.log.info('completed the ``set_master_obj_types`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*Clean up the database tables used by sherlock - maintainance tools*

:Author:
    David Young

:Date Created:
    August 20, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython

# xdocopt-usage-tempx
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from dryxPython import mysql as dms
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
# from ..__init__ import *


def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]


def main(arguments=None):
    """
    *The main function used when ``cleanup_database_tables.py`` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="DEBUG",
        options_first=False
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # unpack remaining cl arguments using `exec` to setup the variable names
    # automatically
    for arg, val in arguments.iteritems():
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        if isinstance(val, str) or isinstance(val, unicode):
            exec(varname + " = '%s'" % (val,))
        else:
            exec(varname + " = %s" % (val,))
        if arg == "--dbConn":
            dbConn = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE cleanup_database_tables.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in locals() and interactiveFlag:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    # call the worker function
    # x-if-settings-or-database-credientials
    cleanup_database_tables(
        log=log,
        settings=settings
    )

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE cleanup_database_tables.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return


###################################################################
# CLASSES                                                         #
###################################################################
class cleanup_database_tables():

    """
    *The worker class for the cleanup_database_tables module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary


    .. todo::

        - @review: when complete, clean cleanup_database_tables class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new 'cleanup_database_tables' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # 2. @flagged: what are the default attrributes each object could have? Add them to variable attribute set here
        # Variable Data Atrributes

        # 3. @flagged: what variable attrributes need overriden in any baseclass(es) used
        # Override Variable Data Atrributes

        # Initial Actions
        from sherlock import database
        db = database(
            log=self.log,
            settings=self.settings
        )
        self.transientsDbConn, self.cataloguesDbConn, self.pmDbConn = db.get()

        return None

    def close(self):
        del self
        return None

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the cleanup_database_tables object*

        **Return:**
            - ``cleanup_database_tables``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        self._update_tcs_helper_catalogue_tables_info_with_new_tables()
        self._updated_row_counts_in_tcs_helper_catalogue_tables_info()
        self._clean_up_columns()
        self._update_tcs_helper_catalogue_views_info_with_new_views()
        self._copy_helper_tables_to_transient_database()

        self.log.info('completed the ``get`` method')
        return cleanup_database_tables

    def _updated_row_counts_in_tcs_helper_catalogue_tables_info(
            self):
        """
        *updated row counts in tcs catalogue tables*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _updated_row_counts_in_tcs_helper_catalogue_tables_info method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``_updated_row_counts_in_tcs_helper_catalogue_tables_info`` method')

        sqlQuery = u"""
            select * from tcs_helper_catalogue_tables_info where table_name like "%%stream" or number_of_rows is null and legacy_table = 0
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        for row in rows:
            tbName = row["table_name"]

            sqlQuery = u"""
                update tcs_helper_catalogue_tables_info set number_of_rows = (select count(*) as count from %(tbName)s) where table_name = "%(tbName)s"
            """ % locals()
            count = dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log,
                quiet=True
            )

        print "Row counts updated in `tcs_helper_catalogue_tables_info` database table"

        sqlQuery = u"""
            select * from tcs_helper_catalogue_views_info where view_name like "%%stream" or number_of_rows is null and legacy_view = 0
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        for row in rows:
            tbName = row["view_name"]

            sqlQuery = u"""
                update tcs_helper_catalogue_views_info set number_of_rows = (select count(*) as count from %(tbName)s) where view_name = "%(tbName)s"
            """ % locals()
            count = dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log,
                quiet=True
            )

        self.log.info(
            'completed the ``_updated_row_counts_in_tcs_helper_catalogue_tables_info`` method')
        return None

    # use the tab-trigger below for new method
    def _update_tcs_helper_catalogue_tables_info_with_new_tables(
            self):
        """
        *update tcs helper catalogue tables info with new tables*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_tcs_helper_catalogue_tables_info_with_new_tables method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``_update_tcs_helper_catalogue_tables_info_with_new_tables`` method')

        sqlQuery = u"""
            SELECT max(id) as thisId FROM tcs_helper_catalogue_tables_info;
        """ % locals()
        thisId = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        try:
            highestId = thisId[0]["thisId"] + 1
        except:
            highestId = 1

        sqlQuery = u"""
            SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA='crossmatch_catalogues' and TABLE_NAME like "tcs_cat%%";
        """ % locals()
        tablesInDatabase = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        sqlQuery = u"""
            SELECT table_name FROM tcs_helper_catalogue_tables_info;
        """ % locals()
        tableList = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        tbList = []
        for tb in tableList:
            tbList.append(tb["table_name"])

        for tb in tablesInDatabase:
            if tb["TABLE_NAME"] not in tbList:
                thisTableName = tb["TABLE_NAME"]
                print "`%(thisTableName)s` added to `tcs_helper_catalogue_tables_info` database table" % locals()
                sqlQuery = u"""
                    INSERT INTO tcs_helper_catalogue_tables_info (
                            id,
                            table_name
                        )
                        VALUES (
                            %(highestId)s,
                            "%(thisTableName)s"
                    )""" % locals()
                dms.execute_mysql_write_query(
                    sqlQuery=sqlQuery,
                    dbConn=self.cataloguesDbConn,
                    log=self.log
                )
                highestId += 1

        self.log.info(
            'completed the ``_update_tcs_helper_catalogue_tables_info_with_new_tables`` method')
        return None

    # use the tab-trigger below for new method
    def _clean_up_columns(
            self):
        """
        *clean up columns*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _clean_up_columns method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_clean_up_columns`` method')

        sqlQueries = [
            "update tcs_helper_catalogue_tables_info set old_table_name = table_name where old_table_name is null;",
            "update tcs_helper_catalogue_tables_info set version_number = 'stream' where table_name like '%%stream' and version_number is null;",
            """update tcs_helper_catalogue_tables_info set in_ned = 0 where table_name like '%%stream' and in_ned is null;""",
            """update tcs_helper_catalogue_tables_info set vizier_link = 0 where table_name like '%%stream' and vizier_link is null;""",
            "update tcs_helper_catalogue_views_info set old_view_name = view_name where old_view_name is null;",
        ]

        for sqlQuery in sqlQueries:
            dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )

        # VIEW OBJECT TYPES
        sqlQuery = u"""
            SELECT view_name FROM crossmatch_catalogues.tcs_helper_catalogue_views_info where legacy_view = 0 and object_type is null;
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        for row in rows:
            view_name = row["view_name"]
            object_type = view_name.replace("tcs_view_", "").split("_")[0]

            sqlQuery = u"""
                update tcs_helper_catalogue_views_info set object_type = "%(object_type)s" where view_name = "%(view_name)s"
            """ % locals()
            dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )

        # MASTER TABLE ID FOR VIEWS
        sqlQuery = u"""
            SELECT view_name FROM crossmatch_catalogues.tcs_helper_catalogue_views_info where legacy_view = 0 and table_id is null;
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        for row in rows:
            view_name = row["view_name"]
            table_name = view_name.replace("tcs_view_", "").split("_")[1:]
            table_name = ("_").join(table_name)
            table_name = "tcs_cat_%(table_name)s" % locals()

            # print table_name

            sqlQuery = u"""
                update tcs_helper_catalogue_views_info set table_id = (select id from tcs_helper_catalogue_tables_info where table_name = "%(table_name)s") where view_name = "%(view_name)s"
            """ % locals()
            dms.execute_mysql_write_query(
                sqlQuery=sqlQuery,
                dbConn=self.cataloguesDbConn,
                log=self.log
            )

        self.log.info('completed the ``_clean_up_columns`` method')
        return None

    # use the tab-trigger below for new method
    def _update_tcs_helper_catalogue_views_info_with_new_views(
            self):
        """
        *update tcs helper catalogue tables info with new tables*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_tcs_helper_catalogue_views_info_with_new_views method
            - @review: when complete add logging
        """
        self.log.info(
            'starting the ``_update_tcs_helper_catalogue_views_info_with_new_views`` method')

        sqlQuery = u"""
            SELECT max(id) as thisId FROM tcs_helper_catalogue_views_info;
        """ % locals()
        thisId = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        try:
            highestId = thisId[0]["thisId"] + 1
        except:
            highestId = 1

        sqlQuery = u"""
            SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='VIEW' AND TABLE_SCHEMA='crossmatch_catalogues' and TABLE_NAME like "tcs_view%%" and TABLE_NAME not like "%%helper%%";
        """ % locals()
        tablesInDatabase = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        sqlQuery = u"""
            SELECT view_name FROM tcs_helper_catalogue_views_info;
        """ % locals()
        tableList = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        tbList = []
        for tb in tableList:
            tbList.append(tb["view_name"])

        for tb in tablesInDatabase:
            if tb["TABLE_NAME"] not in tbList:
                thisViewName = tb["TABLE_NAME"]
                print "`%(thisViewName)s` added to `tcs_helper_catalogue_views_info` database table" % locals()
                sqlQuery = u"""
                    INSERT INTO tcs_helper_catalogue_views_info (
                            id,
                            view_name
                        )
                        VALUES (
                            %(highestId)s,
                            "%(thisViewName)s"
                    )""" % locals()
                dms.execute_mysql_write_query(
                    sqlQuery=sqlQuery,
                    dbConn=self.cataloguesDbConn,
                    log=self.log
                )
                highestId += 1

        self.log.info(
            'completed the ``_update_tcs_helper_catalogue_views_info_with_new_views`` method')
        return None

    # use the tab-trigger below for new method
    def _copy_helper_tables_to_transient_database(
            self):
        """ copy helper tables to transient database

        **Key Arguments:**
            # -

        **Return:**
            - None

        **Todo**
            - @review: when complete, clean _copy_helper_tables_to_transient_database method
            - @review: when complete add logging

        .. todo ::

            - need to change pessto marshall database to database table from settings file
        """
        self.log.info(
            'starting the ``_copy_helper_tables_to_transient_database`` method')

        sqlQuery = u"""
            DELETE FROM pessto_marshall.tcs_helper_catalogue_tables_info;
            INSERT INTO pessto_marshall.tcs_helper_catalogue_tables_info SELECT * FROM crossmatch_catalogues.tcs_helper_catalogue_tables_info;
        """ % locals()
        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.transientsDbConn,
            log=self.log,
            quiet=False,
            Force=False,  # do not exit code if error occurs
            manyValueList=False  # a list of value tuples if executing more than one insert
        )

        sqlQuery = u"""
            DELETE FROM pessto_marshall.tcs_helper_catalogue_views_info;
            INSERT INTO pessto_marshall.tcs_helper_catalogue_views_info SELECT * FROM crossmatch_catalogues.tcs_helper_catalogue_views_info;
        """ % locals()
        dms.execute_mysql_write_query(
            sqlQuery=sqlQuery,
            dbConn=self.transientsDbConn,
            log=self.log,
            quiet=False,
            Force=False,  # do not exit code if error occurs
            manyValueList=False  # a list of value tuples if executing more than one insert
        )

        self.log.info(
            'completed the ``_copy_helper_tables_to_transient_database`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx

# xt-class-tmpx


###################################################################
# PUBLIC FUNCTIONS                                                #
###################################################################
# xt-worker-def

# use the tab-trigger below for new function
# xt-def-with-logger

###################################################################
# PRIVATE (HELPER) FUNCTIONS                                      #
###################################################################

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
update_ned_stream.py
==================
:Summary:
    Update the NED stream by querying NED with an area larger than required for the transients listed

:Author:
    David Young

:Date Created:
    November  2, 2015

:dryx syntax:
    - ``_someObject`` = a 'private' object that should only be changed for debugging

:Notes:
    - If you have any questions requiring this script/module please email me: davidrobertyoung@gmail.com

:Tasks:
    @review: when complete pull all general functions and classes into dryxPython
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from dryxPython import mysql as dms
from sherlock.imports.ned_conesearch import ned_conesearch
from HMpTy import htm
import numpy as np
from cStringIO import StringIO


class update_ned_stream():
    """
    The worker class for the update_ned_stream module

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``transientsMetadataList`` -- the metadata for the transients (list of dictionaries, must contain keys 'ra' and 'dec')
        - ``cataloguesDbConn`` -- database connection for the catalogue database

    **Todo**
        - @review: when complete, clean update_ned_stream class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
        self,
        log,
        cataloguesDbConn,
        settings=False,
        transientsMetadataList=[]
    ):
        self.log = log
        log.debug("instansiating a new 'update_ned_stream' object")
        self.settings = settings
        self.transientsMetadataList = transientsMetadataList
        self.cataloguesDbConn = cataloguesDbConn

        # xt-self-arg-tmpx

        # CHECK SETTINGS FOR NED SEARCH - EXIT IF NONE FOUND
        self.update = False
        for search in self.settings["search algorithm"]:
            if "ned" in self.settings["search algorithm"][search]["database table"]:
                self.update = True

        # Initial Actions

        return None

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """get the update_ned_stream object

        **Return:**
            - ``update_ned_stream``

        **Todo**
            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        if self.update == False:
            return None

        coordinateList = []
        for i in self.transientsMetadataList:
            thisList = str(i["ra"]) + " " + str(i["dec"])
            coordinateList.append(thisList)

        initialListLen = len(coordinateList)
        coordinateList = self._remove_previous_ned_queries(
            coordinateList=coordinateList
        )
        newListLen = len(coordinateList)
        previousSearches = initialListLen - newListLen

        print "%(newListLen)s/%(initialListLen)s coordinate sets need an inital NED conesearch to find matched source names (%(previousSearches)s coordinate sets found in search history)" % locals()

        this = ned_conesearch(
            log=self.log,
            settings=self.settings,
            coordinateList=coordinateList,
            cataloguesDbConn=self.cataloguesDbConn
        )
        this.get()

        self.log.info('completed the ``get`` method')
        return update_ned_stream

    def _remove_previous_ned_queries(
            self,
            coordinateList):
        """ remove previous ned queries

        **Key Arguments:**
            # - ``coordinateList`` -- set of coordinate to check for previous queries

        **Return:**
            - ``updatedCoordinateList`` -- coordinate list with previous queries removed
        """
        self.log.info('starting the ``_remove_previous_ned_queries`` method')

        # IMPORTS
        import math
        from dryxPython import astrotools as dat
        from datetime import datetime, date, time, timedelta

        print "Removing transient/subdisk areas that have had previous searches performed"

        # VARAIBLES
        self.mesh16 = htm.HTM(16)
        updatedCoordinateList = []

        #  FIND THE LARGE NED QUERY DONE SO FAR FROM DATABASE
        sqlQuery = u"""
            select arcsecRadius from tcs_helper_ned_query_history order by arcsecRadius desc limit 1
        """ % locals()
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )
        largestNedQueryDeg = rows[0]["arcsecRadius"] / 3600.

        # CALCULATE THE OLDEST RESULTS LIMIT
        now = datetime.now()
        td = timedelta(
            days=self.settings["ned stream refresh rate in days"])
        refreshLimit = now - td
        refreshLimit = refreshLimit.strftime("%Y-%m-%d %H:%M:%S")

        # FOR EACH TRANSIENT IN COORDINATE LIST ...
        totalCount = len(coordinateList)
        count = 0
        htmWhereClauses = []
        transRas = []
        transDecs = []
        for c in coordinateList:
            count += 1
            if count > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            if count > totalCount:
                count = totalCount
            percent = (float(count) / float(totalCount)) * 100.
            print "  %(count)s / %(totalCount)s (%(percent)1.1f%%) transients/subdisks checked" % locals()
            this = c.split(" ")
            raDeg = float(this[0])
            decDeg = float(this[1])
            transRas.append(raDeg)
            transDecs.append(decDeg)

            # BUILD WHERE SECTION OF CLAUSE
            # CREATE AN ARRAY OF RELEVANT HTMIDS AND FIND MAX AND MIN
            thisArray = self.mesh16.intersect(
                raDeg, decDeg, largestNedQueryDeg, inclusive=True)
            hmax = thisArray.max()
            hmin = thisArray.min()
            htmWhereClause = "(htm16ID between %(hmin)s and %(hmax)s)" % locals(
            )
            htmWhereClauses.append(htmWhereClause)

        htmWhereClauses = " or ".join(htmWhereClauses)

        # DO A LARGE FIRST SWEEP FOR PREVIOUS SEARCHES - BEFORE PINPOINTING
        if not len(htmWhereClauses):
            return []
        sqlQuery = "select * from tcs_helper_ned_query_history where (%(htmWhereClauses)s) and dateQueried > '%(refreshLimit)s'" % locals(
        )
        rows = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        nedRas = []
        nedDecs = []
        nedRadii = []
        for row in rows:
            nedRas.append(row["raDeg"])
            nedDecs.append(row["decDeg"])
            nedRadii.append(row["arcsecRadius"])

        # CREATE TWO ARRAYS OF RA,DEC (1. SUBDISKS & 2. NED PREVIOUS QUERY
        # RETURNS)
        tRa = np.array(transRas)
        tDec = np.array(transDecs)
        raList = np.array(nedRas)
        decList = np.array(nedDecs)

        indexList1, indexList2, separation = self.mesh16.match(
            tRa, tDec, raList, decList, largestNedQueryDeg, maxmatch=0)

        matchList = [False] * len(transRas)
        for i in xrange(indexList1.size):
            if separation[i] * 3600. + self.settings["first pass ned search radius arcec"] < nedRadii[indexList2[i]]:
                matchList[indexList1[i]] = True

        for m, c in zip(matchList, coordinateList):
            if m == False:
                updatedCoordinateList.append(c)

        self.log.info('completed the ``_remove_previous_ned_queries`` method')
        return updatedCoordinateList

    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx



#!/usr/local/bin/python
# encoding: utf-8
"""
*The transient to catalogue sources crossmatcher for sherlock*

:Author:
    David Young

:Date Created:
    July 1, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import math
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import astrotools as dat
from dryxPython import commonutils as dcu
from fundamentals import tools, times


class crossmatcher():

    """
    *The worker class for the crossmatcher module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection for the catalogues
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``colMaps`` -- maps of the important column names for each table/view in the crossmatch-catalogues database
        - ``transients`` -- the list of transients
    """
    # INITIALISATION

    def __init__(
            self,
            log,
            dbConn=False,
            settings=False,
            colMaps=False,
            transients=[]
    ):
        self.log = log
        log.debug("instansiating a new '_crossmatcher' object")
        self.dbConn = dbConn
        self.settings = settings
        self.transients = transients
        self.colMaps = colMaps
        # xt-self-arg-tmpx

        return None

    # METHOD ATTRIBUTES
    def get(self):
        """
        *get the crossmatcher object*

        **Return:**
            - ``classifications`` - list of all classifications
        """
        self.log.debug('starting the ``get`` method')

        classifications = self._crossmatch_transients_against_catalogues()

        self.log.debug('completed the ``get`` method')
        return classifications

    def _crossmatch_transients_against_catalogues(
            self):
        """
        *crossmatch transients against catalogues*
        """

        classifications = []

        self.log.debug(
            'starting the ``_crossmatch_transients_against_catalogues`` method')

        # COUNT NUMBER OF TRANSIENT TO CROSSMATCH
        classification = []
        numberOfTransients = len(self.transients)
        count = 0

        # FOR EACH TRANSIENT SOURCE IN THE LIST ...
        for thisIndex, transient in enumerate(self.transients):

            tId = transient['id']
            tName = transient['name']

            # PRINT A STATUS UPDATE
            count += 1
            if count > 1:
                # CURSOR UP ONE LINE AND CLEAR LINE
                sys.stdout.write("\x1b[1A\x1b[2K")
            thisCount = thisIndex + 1
            message = "Classifying object `%(tName)s` [%(thisCount)s/%(numberOfTransients)d]" % locals(
            )
            print message
            self.log.info('\n\n\nmessage: %(message)s' % locals())

            # RESET MATCH VALUES
            match = False  # SET TO TRUE WHENEVER A MATCH IS FIRST FOUND
            # THE LIST OF OBJECTS FOUND ASSOCIATED WITH THE SOURCE
            matches = []
            allMatches = []
            # SET TO TRUE WHENEVER WE WANT TO TERMINATE THE SEARCH
            searchDone = False
            self.stopAlgoritm = False

            # GRAB SEARCH ALGORITHM AND ITERATE THROUGH IT IN ORDER
            # PRESENTED IN THE SETTINGS FILE
            sa = self.settings["search algorithm"]
            for searchName, searchPara in sa.iteritems():
                self.log.info("""  searching: %(searchName)s""" % locals())
                if "physical radius kpc" in searchPara:
                    # THE PHYSICAL SEPARATION SEARCHES
                    self.log.info(
                        'checking physical distance crossmatches in %(searchName)s' % locals())
                    searchDone, matches = self._physical_separation_search(
                        objectRow=transient,
                        searchPara=searchPara,
                        searchName=searchName
                    )
                else:
                    # THE ANGULAR SEPARATION SEARCHES
                    self.log.info(
                        'Crossmatching against %(searchName)s' % locals())
                    searchDone, matches = self.searchCatalogue(
                        objectList=[transient],
                        searchPara=searchPara,
                        searchName=searchName
                    )

                # ADD CLASSIFICATION AND CROSSMATCHES IF FOUND
                if searchDone and matches:
                    self.log.info("     match found for %(tName)s" % locals())
                    match = True
                    # @action update objectType -- it gets overwritten with each subsequent search
                    objectType = searchPara["transient classification"]
                    allMatches = allMatches + matches
                    del matches

                    if "stop algorithm on match" in searchPara and searchPara["stop algorithm on match"] == True:
                        self.stopAlgoritm = True
                        self.log.info(
                            "     stopping algorithm for %(tName)s" % locals())
                        break

            # IF NO MATCH IS FOUND THEN WE HAVE AN 'ORPHAN'
            if not match:
                self.log.info(
                    "   %(tName)s classified as an orphan" % locals())
                objectType = "ORPHAN"

            # PERFORM ANY SUPPLIMENTARY SEARCHES
            if "supplementary search" in self.settings:
                ss = self.settings["supplementary search"]
                for searchName, searchPara in ss.iteritems():
                    self.log.info("""  searching: %(searchName)s""" % locals())
                    searchDone, supMatches = self.searchCatalogue(
                        objectList=[transient],
                        searchPara=searchPara,
                        searchName=searchName
                    )
                    if searchDone and supMatches:
                        self.log.info(
                            "     match found for %(tName)s" % locals())
                        objectType = objectType + \
                            searchPara["transient classification"]
                        allMatches = allMatches + supMatches
                        del supMatches

            # ADD DETAILS TO THE lOGS
            oldClass = transient['object_classification']
            # oldClass = self._lookup_classification_dict(
            #     flag=transient['object_classification'])
            # newClass = self._lookup_classification_dict(
            #     flag=objectType)
            # self.log.info(
            #     """%(transient)s:  *** Object ID = %(tId)s (%(tName)s): CLASSIFICATION = %(newClass)s (PREVIOUSLY = %(oldClass)s)""" % locals())

            # BUILD THE LIST OF CROSSMATCH DICTIONARIES AND ADD THEM THE THE
            # LIST OF ALL SOURCE CLASSIFICATIONS
            crossmatches = []
            if allMatches:
                inputRow = allMatches[0][0]
                for m in allMatches:
                    catalogueTableName = m[2]
                    thisObjectType = m[3]
                    catalogueTableId = self.colMaps[
                        catalogueTableName]["table_id"]
                    catalogueViewId = self.colMaps[
                        catalogueTableName]["view_id"]

                    for row in m[1]:
                        crossmatch = {}
                        crossmatch["transientObjectId"] = inputRow["id"]
                        crossmatch["catalogueObjectId"] = row[1][
                            self.colMaps[catalogueTableName]["objectNameColName"]]
                        crossmatch["catalogueTableId"] = catalogueTableId
                        crossmatch["catalogueViewId"] = catalogueViewId
                        crossmatch["catalogueTableName"] = self.colMaps[
                            catalogueTableName]["table_name"]
                        crossmatch["catalogueViewName"] = self.colMaps[
                            catalogueTableName]["view_name"]
                        crossmatch["separation"] = row[0]
                        crossmatch["z"] = row[1]["xmz"]
                        crossmatch["scale"] = row[1]["xmscale"]
                        crossmatch["distance"] = row[1]["xmdistance"]
                        crossmatch["distanceModulus"] = row[
                            1]["xmdistanceModulus"]
                        crossmatch["association_type"] = thisObjectType
                        crossmatch["sourceType"] = row[1]["sourceType"]
                        crossmatch["sourceSubType"] = row[1]["sourceSubType"]
                        crossmatch["searchName"] = row[1]["searchName"]
                        crossmatch["xmdirectdistance"] = row[
                            1]["xmdirectdistance"]
                        crossmatch["xmdirectdistanceModulus"] = row[
                            1]["xmdirectdistanceModulus"]
                        crossmatch["xmdirectdistancescale"] = row[
                            1]["xmdirectdistancescale"]
                        crossmatch["originalSearchRadius"] = row[
                            1]["originalSearchRadius"]
                        crossmatch["sourceRa"] = row[
                            1]["sourceRa"]
                        crossmatch["sourceDec"] = row[
                            1]["sourceDec"]
                        crossmatch["xmmajoraxis"] = row[1]["xmmajoraxis"]
                        if crossmatch["sourceSubType"] == "null":
                            crossmatch["sourceSubType"] = None
                        if "physical_separation_kpc" in row[
                                1].keys():
                            crossmatch["physical_separation_kpc"] = row[
                                1]["physical_separation_kpc"]
                        else:
                            crossmatch["physical_separation_kpc"] = None
                        crossmatches.append(crossmatch)

            # self.log.info('crossmatches: %(crossmatches)s' % locals())

            classification = {'id': transient['id'], 'object_classification_old': transient[
                'object_classification'], 'object_classification_new': objectType, 'crossmatches': crossmatches}
            classifications.append(classification)

        self.log.debug(
            'completed the ``_crossmatch_transients_against_catalogues`` method')
        return classifications

    def searchCatalogue(self, objectList, searchPara={}, searchName=""):
        """*Cone Search wrapper to make it a little more user friendly*"""

        from sherlock import conesearcher

        message = "  Starting `%(searchName)s` catalogue conesearch" % locals(
        )
        # print message

        # EXTRACT PARAMETERS FROM ARGUMENTS & SETTINGS FILE
        if "physical radius kpc" in searchPara:
            physicalSearch = True
            searchName = searchName + " physical"
        else:
            physicalSearch = False
            searchName = searchName + " angular"
        radius = searchPara["angular radius arcsec"]
        catalogueName = searchPara["database table"]
        matchedType = searchPara["transient classification"]

        # VARIABLES
        matchedObjects = []
        matchSubset = []
        searchDone = True

        # FAINT & BRIGHT STAR EXTRAS
        try:
            faintMagColumn = searchPara["faint mag column"]
            faintLimit = searchPara["faint limit"]
        except:
            faintMagColumn = False
            faintLimit = False
        try:
            filterColumns = searchPara["filter names"]
            magColumns = searchPara["mag columns"]
            magErrColumns = searchPara["mag err columns"]
        except:
            filterColumns = False
            magColumns = False
            magErrColumns = False

        totalCount = len(objectList)
        count = 0
        for row in objectList:
            count += 1
            if count > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            if count > totalCount:
                count = totalCount
            percent = (float(count) / float(totalCount)) * 100.
            cs = conesearcher(
                log=self.log,
                ra=row['ra'],
                dec=row['dec'],
                radius=radius,
                colMaps=self.colMaps,
                tableName=catalogueName,
                queryType=2,
                dbConn=self.dbConn,
                settings=self.settings,
                physicalSearch=physicalSearch,
                transType=searchPara["transient classification"]
            )
            message, xmObjects = cs.get()
            del cs
            resultLen = len(xmObjects)
            print "  %(count)s / %(totalCount)s (%(percent)1.1f%%) transients conesearched against with %(searchName)s - %(resultLen)s sources matched" % locals()

            # DID WE SEARCH THE CATALOGUES CORRECTLY?
            if message and (message.startswith('Error') or 'not recognised' in message):
                # SUCCESSFUL CONE SEARCHES SHOULD NOT RETURN AN ERROR MESSAGE,
                # OTHERWISE SOMETHING WENT WRONG.
                print "Database error - cone search unsuccessful.  Message was:"
                print "\t%s" % message
                searchDone = False

            if xmObjects:
                numberOfMatches = len(xmObjects)
                nearestSep = xmObjects[0][0]
                nearestCatRow = xmObjects[0][1]
                nearestCatId = nearestCatRow[
                    self.colMaps[catalogueName]["objectNameColName"]]

                # CALCULATE PHYSICAL PARAMETERS ... IF WE CAN
                for xm in xmObjects:
                    redshift = None
                    xmz = None
                    xmscale = None
                    xmdistance = None
                    xmdistanceModulus = None
                    xmmajoraxis = None
                    xmdirectdistance = None
                    xmdirectdistancescale = None
                    xmdirectdistanceModulus = None

                    # IF THERE'S A REDSHIFT, CALCULATE PHYSICAL PARAMETERS
                    if self.colMaps[catalogueName]["redshiftColName"]:
                        # THE CATALOGUE HAS A REDSHIFT COLUMN
                        redshift = xm[1][
                            self.colMaps[catalogueName]["redshiftColName"]]
                    if redshift and redshift > 0.0:
                        # CALCULATE DISTANCE MODULUS, ETC
                        redshiftInfo = dat.convert_redshift_to_distance(
                            z=redshift
                        )
                        if redshiftInfo:
                            xmz = redshiftInfo['z']
                            xmscale = redshiftInfo['da_scale']
                            xmdistance = redshiftInfo['dl_mpc']
                            xmdistanceModulus = redshiftInfo['dmod']
                    # ADD MAJOR AXIS VALUE
                    if "or within semi major axis" in searchPara and searchPara["or within semi major axis"] == True and self.colMaps[catalogueName]["semiMajorColName"] and xm[1][self.colMaps[catalogueName]["semiMajorColName"]]:
                        xmmajoraxis = xm[1][
                            self.colMaps[catalogueName]["semiMajorColName"]] * self.colMaps[catalogueName]["semiMajorToArcsec"]
                    # ADD DISTANCE VALUES
                    if self.colMaps[catalogueName]["distanceColName"] and xm[1][self.colMaps[catalogueName]["distanceColName"]]:
                        xmdirectdistance = xm[1][
                            self.colMaps[catalogueName]["distanceColName"]]
                        xmdirectdistancescale = xmdirectdistance / 206.264806
                        xmdirectdistanceModulus = 5 * \
                            math.log10(xmdirectdistance * 1e6) - 5
                    xm[1]['xmz'] = xmz
                    xm[1]['xmscale'] = xmscale
                    xm[1]['xmdistance'] = xmdistance
                    xm[1]['xmdistanceModulus'] = xmdistanceModulus
                    xm[1]['xmmajoraxis'] = xmmajoraxis
                    xm[1]['xmdirectdistance'] = xmdirectdistance
                    xm[1]['xmdirectdistancescale'] = xmdirectdistancescale
                    xm[1]['xmdirectdistanceModulus'] = xmdirectdistanceModulus

                # GRAB SOURCE TYPES
                for xm in xmObjects:
                    subType = None
                    # IF THERE'S A SUBTYPE - ADD IT
                    if self.colMaps[catalogueName]["subTypeColName"]:
                        # THE CATALOGUE HAS A REDSHIFT COLUMN
                        subType = xm[1][
                            self.colMaps[catalogueName]["subTypeColName"]]
                        if subType == "null":
                            subType = None

                    xm[1]['sourceSubType'] = subType
                    xm[1]['sourceType'] = self.colMaps[
                        catalogueName]["object_type"]
                    xm[1]["searchName"] = searchName
                    xm[1]["sourceRa"] = xm[1][
                        self.colMaps[catalogueName]["raColName"]]
                    xm[1]["sourceDec"] = xm[1][
                        self.colMaps[catalogueName]["decColName"]]
                    xm[1]["originalSearchRadius"] = radius

                    # SOURCE MAG AND FILTER - CHOOSE MAG WITH LOWEST ERROR

                    xm[1]["sourceFilter"] = self.colMaps[
                        catalogueName]["filterName1ColName"]

                    xm[1]["sourceMagnitude"] = xm[1][
                        self.colMaps[catalogueName]["filter1ColName"]]
                    if self.colMaps[catalogueName]["filterErr1ColName"]:
                        xm[1]["sourceMagnitudeErr"] = xm[1][
                            self.colMaps[catalogueName]["filterErr1ColName"]]
                    else:
                        xm[1]["sourceMagnitudeErr"] = None
                    thisMagErr = xm[1]["sourceMagnitude"]
                    index = 2
                    while thisMagErr is not None and index < 6:
                        if self.colMaps[catalogueName][
                                "filterErr%(index)sColName" % locals()]:
                            thisMagErr = xm[1][self.colMaps[catalogueName][
                                "filterErr%(index)sColName" % locals()]]
                        else:
                            thisMagErr = None
                        if thisMagErr is not None and thisMagErr < xm[1]["sourceMagnitudeErr"]:
                            filterName = self.colMaps[
                                catalogueName]["filterName%(index)sColName" % locals()][:4]
                            if "col_" not in filterName:
                                xm[1]["sourceFilter"] = filterName
                            else:
                                xm[1]["sourceFilter"] = xm[1][filterName]
                            xm[1]["sourceMagnitude"] = xm[1][
                                self.colMaps[catalogueName]["filter%(index)sColName" % locals()]]
                            xm[1]["sourceMagnitudeErr"] = thisMagErr
                        index += 1

                matchedObjects.append(
                    [row, xmObjects, catalogueName, matchedType])

        # FAINT STAR CUTS
        if searchDone and matchedObjects and faintMagColumn:
            faintStarMatches = []
            matchSubset = []
            for row in matchedObjects[0][1]:
                rMag = row[1][faintMagColumn]
                if rMag and rMag > faintLimit:
                    matchSubset.append(row)

            if matchSubset:
                faintStarMatches.append(
                    [matchedObjects[0][0], matchSubset, matchedObjects[0][2], matchedObjects[0][3]])
            matchedObjects = faintStarMatches
        # BRIGHT STAR MATCH
        elif "tcs_view_star_sdss" in catalogueName:
            sdssStarMatches = []
            matchSubset = []

            if searchDone and matchedObjects:
                for row in matchedObjects[0][1]:
                    rMag = row[1]["petroMag_r"]
                    separation = row[0]
                    # Line passes through (x,y) = (2.5,18) and (19,13)
                    lineTwo = -((18 - 13) / (19 - 2.5)) * \
                        separation + 13 + 19 * ((18 - 13) / (19 - 2.5))
                    if rMag < 13.0:
                        matchSubset.append(row)
                    elif rMag >= 13.0 and rMag < 18.0:
                        if rMag < lineTwo:
                            matchSubset.append(row)
                    elif rMag >= 18.0 and separation < 2.5:
                        matchSubset.append(row)
            if matchSubset:
                sdssStarMatches.append(
                    [matchedObjects[0][0], matchSubset, matchedObjects[0][2], matchedObjects[0][3]])
            matchedObjects = sdssStarMatches

        if "match nearest source only" in searchPara and searchPara["match nearest source only"] == True and len(matchedObjects):
            matchedObjects = [[matchedObjects[0][0], [
                matchedObjects[0][1][0]], matchedObjects[0][2], matchedObjects[0][3]]]

        message = "  Finished `%(searchName)s` catalogue conesearch" % locals(
        )
        # print message

        return searchDone, matchedObjects

    def _lookup_classification_dict(
            self,
            flag):
        """
        *append reversed classification dictionary*

        **Key Arguments:**
            # -

        **Return:**
            - classifications
        """
        self.log.debug(
            'starting the ``_append_reversed_classification_dictionary`` method')

        flagTypes = []

        if flag == 0:
            return 'unclassified'

        for i in range(16):
            mask = (1 << 15) >> i

            try:
                result = flag & mask
                if result:
                    for code in self.settings["classifications"]:
                        if self.settings["classifications"][code]["flag"] == result:
                            flagTypes.append(
                                self.settings["classifications"][code]["human"])
            except TypeError:
                # We got a None (i.e. NULL) valueflags
                return ''

        self.log.debug(
            'completed the ``_append_reversed_classification_dictionary`` method')
        return ' + '.join(flagTypes)

    def _physical_separation_search(self, objectRow, searchPara, searchName):
        """
        *physical separation search*

        **Key Arguments:**
            - ``objectRow`` -- transient to be crossmatched
            - ``searchPara`` -- parameters of the search (from settings file)
            - ``searchName`` -- the name of the search

        **Return:**
            - searchDone -- did search complete successfully
            - matchedObjects -- any sources matched against the object
        """
        self.log.debug('starting the ``_physical_separation_search`` method')

        # SETUP PARAMETERS
        tableName = searchPara["database table"]
        angularRadius = searchPara["angular radius arcsec"]
        matchedObjects = []
        matchSubset = []
        physicalRadius = searchPara["physical radius kpc"]
        matchedType = searchPara["transient classification"]

        # RETURN ALL ANGULAR MATCHES BEFORE RETURNING NEAREST PHYSICAL SEARCH
        nearestOnly = searchPara["match nearest source only"]
        tmpSearchPara = dict(searchPara)
        tmpSearchPara["match nearest source only"] = False

        # ANGULAR CONESEARCH ON CATALOGUE
        searchDone, matches = self.searchCatalogue(
            objectList=[objectRow],
            searchPara=tmpSearchPara,
            searchName=searchName
        )
        if "physical radius kpc" in searchPara:
            physicalSearch = True
            searchName = searchName + " physical"
        else:
            physicalSearch = False
            searchName = searchName + " angular"

        # OK - WE HAVE SOME ANGULAR SEPARATION MATCHES. NOW SEARCH THROUGH THESE FOR MATCHES WITH
        # A PHYSICAL SEPARATION WITHIN THE PHYSICAL RADIUS.
        if searchDone and matches:
            objectName = matches[0][0]["name"]
            for row in matches[0][1]:

                thisMatch = False
                physicalSeparation = None
                newSearchName = searchName

                # CALCULATE MOST ACCURATE PHYSICAL SEPARATION
                if row[1]["xmdirectdistancescale"]:
                    physicalSeparation = row[1][
                        "xmdirectdistancescale"] * row[0]
                elif row[1]["xmscale"]:
                    physicalSeparation = row[1]["xmscale"] * row[0]

                # FIRST CHECK FOR MAJOR AXIS MEASUREMENT
                if row[1]["xmmajoraxis"] and row[0] < row[1]["xmmajoraxis"] * self.settings["galaxy radius stetch factor"]:
                    self.log.info(
                        '%(objectName)s is found within the major axis measurement of the source' % locals())
                    thisMatch = True
                    newSearchName = newSearchName + \
                        " (within %s * major axis)" % (
                            self.settings["galaxy radius stetch factor"],)
                    newAngularSep = row[1][
                        "xmmajoraxis"] * self.settings["galaxy radius stetch factor"]
                # NOW CHECK FOR A DIRECT DISTANCE MEASUREMENT
                elif row[1]["xmdirectdistancescale"] and physicalSeparation < physicalRadius:
                    self.log.info(
                        '%(objectName)s is found within %(physicalSeparation)s Kpc of the source centre (direct distance measurement)' % locals())
                    thisMatch = True
                    newSearchName = newSearchName + " (direct distance)"
                    newAngularSep = physicalRadius / \
                        row[1]["xmdirectdistancescale"]
                # NEW CHECK FOR A REDSHIFT DISTANCE
                elif row[1]["xmscale"] and physicalSeparation < physicalRadius:
                    self.log.info(
                        '%(objectName)s is found within %(physicalSeparation)s Kpc of the source centre (redshift distance measurement)' % locals())
                    thisMatch = True
                    newSearchName = newSearchName + " (redshift distance)"
                    newAngularSep = physicalRadius / row[1]["xmscale"]
                else:
                    self.log.info(
                        '%(objectName)s is not associated with the source (from inspecting the transient/source physical separation calculations)' % locals())

                if thisMatch == True:
                    row[1]["physical_separation_kpc"] = physicalSeparation
                    row[1]["originalSearchRadius"] = newAngularSep
                    if physicalSeparation:
                        self.log.info(
                            "\t\tPhysical separation = %.2f kpc" % (physicalSeparation,))
                    row[1]["searchName"] = newSearchName
                    matchSubset.append([physicalSeparation, row])

        if matchSubset:
            from operator import itemgetter
            matchSubset = sorted(matchSubset, key=itemgetter(0))

            if nearestOnly == True:
                theseMatches = [matchSubset[0][1]]
            else:
                theseMatches = []
                for i in matchSubset:
                    theseMatches.append(i[1])

            matchedObjects.append(
                [matches[0][0], theseMatches, matches[0][2], matchedType])
        self.log.debug('completed the ``_physical_separation_search`` method')

        return searchDone, matchedObjects

    # use the tab-trigger below for new method
    # xt-class-method



"""
***********************
commonutils sub-package
***********************

:Summary:
    A collection of miscellaneous tools used throughout the Sherlock package

"""


from update_wiki_pages import update_wiki_pages



#!/usr/local/bin/python
# encoding: utf-8
"""
*Import veron catagloue from plain text file*

:Author:
    David Young

:Date Created:
    August 25, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import codecs
import string

import re
from docopt import docopt
from dryxPython import mysql as dms
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from ._base_importer import _base_importer


class veron(_base_importer):

    """
    *The worker class for the veron module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``pathToDataFIle`` -- path to the veron data file
        - ``version`` -- version of the veron catalogue
        - ``catalogueName`` -- the name of the catalogue

    .. todo::

        - @review: when complete, clean veron class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the veron object*

        **Return:**
            - ``veron``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        self.dictList = self.create_dictionary_of_veron()
        self.add_data_to_database_table()
        self.add_htmids_to_database_table()

        self.log.info('completed the ``get`` method')
        return veron

    def create_dictionary_of_veron(
            self):
        """
        *create dictionary of veron*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean create_dictionary_of_veron method
            - @review: when complete add logging
        """
        self.log.info('starting the ``create_dictionary_of_veron`` method')

        dictList = []
        lines = string.split(self.catData, '\n')

        totalCount = len(lines)
        count = 0
        switch = 0
        for line in lines:
            if (len(line) == 0 or line[0] in ["#", " "]) and switch == 0:
                continue
            else:
                switch = 1
            count += 1
            if count > 1:
                # Cursor up one line and clear line
                sys.stdout.write("\x1b[1A\x1b[2K")
            print "%(count)s / %(totalCount)s veron data added to memory" % locals()

            if count == 1:
                theseKeys = []
                someKeys = string.split(line, '|')
                for key in someKeys:
                    if key == "_RAJ2000":
                        key = "raDeg"
                    if key == "_DEJ2000":
                        key = "decDeg"
                    if key == "Cl":
                        key = "class"
                    if key == "nR":
                        key = "not_radio"
                    if key == "Name":
                        key = "name"
                    if key == "l_z":
                        key = "redshift_flag"
                    if key == "z":
                        key = "redshift"
                    if key == "Sp":
                        key = "spectral_classification"
                    if key == "n_Vmag":
                        key = "magnitude_filter"
                    if key == "Vmag":
                        key = "magnitude"
                    if key == "B-V":
                        key = "B_V"
                    if key == "U-B":
                        key = "U_B"
                    if key == "Mabs":
                        key = "abs_magnitude"
                    theseKeys.append(key)
                continue

            if count in [2, 3]:
                continue

            thisDict = {}
            theseValues = string.split(line, '|')

            for k, v in zip(theseKeys, theseValues):
                v = v.strip()
                if len(v) == 0 or v == "-":
                    v = None
                thisDict[k] = v
            dictList.append(thisDict)

        self.log.info(
            'completed the ``create_dictionary_of_veron`` method')
        return dictList

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx

# xt-class-tmpx


###################################################################
# PUBLIC FUNCTIONS                                                #
###################################################################
# xt-worker-def

# use the tab-trigger below for new function
# xt-def-with-logger

###################################################################
# PRIVATE (HELPER) FUNCTIONS                                      #
###################################################################

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*Update the Github wiki pages with useful info*

:Author:
    David Young

:Date Created:
    September 9, 2015

.. todo::
    
    @review: when complete pull all general functions and classes into dryxPython

# xdocopt-usage-tempx
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import mysql as dms
from dryxPython import commonutils as dcu
from fundamentals import tools, times
# from ..__init__ import *


###################################################################
# CLASSES                                                         #
###################################################################
class update_wiki_pages():

    """
    *The worker class for the update_wiki_pages module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary


    .. todo::

        - @review: when complete, clean update_wiki_pages class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation
    # 1. @flagged: what are the unique attrributes for each object? Add them
    # to __init__

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new 'update_wiki_pages' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # 2. @flagged: what are the default attrributes each object could have? Add them to variable attribute set here
        # Variable Data Atrributes

        # 3. @flagged: what variable attrributes need overriden in any baseclass(es) used
        # Override Variable Data Atrributes

        # Initial Actions
        from sherlock import database
        db = database(
            log=self.log,
            settings=self.settings
        )
        self.transientsDbConn, self.cataloguesDbConn, self.pmDbConn = db.get()

        return None

    # 4. @flagged: what actions does each object have to be able to perform? Add them here
    # Method Attributes
    def get(self):
        """
        *get the update_wiki_pages object*

        **Return:**
            - ``update_wiki_pages``

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.info('starting the ``get`` method')

        if "sherlock wiki root" not in self.settings:
            print "Sherlock wiki settings not found in settings file"
            return

        self._get_table_infos()
        self._get_view_infos()
        self._get_stream_view_infos()
        self._create_md_tables()
        self._write_wiki_pages()
        self._update_github()

        # RECODE INTO ASCII
        self.mdTable = self.mdTable.encode("utf-8", "ignore")

        self.log.info('completed the ``get`` method')
        return update_wiki_pages

    def _get_table_infos(
            self):
        """
        *get table infos*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _get_table_infos method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_get_table_infos`` method')

        sqlQuery = u"""
            SELECT * FROM crossmatch_catalogues.tcs_helper_catalogue_tables_info where legacy_table = 0 and table_name not like "legacy%%" order by number_of_rows desc;
        """ % locals()
        self.tableInfo = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info('completed the ``_get_table_infos`` method')
        return None

    def _get_view_infos(
            self):
        """
        *get table infos*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _get_view_infos method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_get_view_infos`` method')

        sqlQuery = u"""
            SELECT * FROM crossmatch_catalogues.tcs_helper_catalogue_views_info where legacy_view = 0 and view_name not like "legacy%%" order by number_of_rows desc;
        """ % locals()
        self.viewInfo = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info('completed the ``_get_view_infos`` method')
        return None

    def _get_stream_view_infos(
            self):
        """
        *get table infos*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _get_view_infos method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_get_stream_view_infos`` method')

        sqlQuery = u"""
            SELECT * FROM crossmatch_catalogues.tcs_helper_catalogue_tables_info where legacy_table = 0 and table_name not like "legacy%%"  and table_name like "%%stream" order by number_of_rows desc;
        """ % locals()
        self.streamInfo = dms.execute_mysql_read_query(
            sqlQuery=sqlQuery,
            dbConn=self.cataloguesDbConn,
            log=self.log
        )

        self.log.info('completed the ``_get_stream_view_infos`` method')
        return None

    # use the tab-trigger below for new method
    def _create_md_tables(
            self):
        """
        *create md table*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _create_md_tables method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_create_md_tables`` method')

        header = u"""
| <sub>Table Name</sub> | <sub>Description</sub> | <sub>Reference</sub> | <sub>Number Rows</sub> | <sub>Vizier</sub> | <sub>NED</sub> | <sub>Objects</sub> | <sub>Weight (1-10)</sub> |
| :--- | :--- | :--- | :--- | :--- | :--- | :--- | :--- |"""

        rows = u""
        for ti in self.tableInfo:
            table_name = ti["table_name"]
            description = ti["description"]
            url = ti["url"]
            number_of_rows = ti["number_of_rows"]
            reference_url = ti["reference_url"]
            reference_text = ti["reference_text"]
            notes = ti["notes"]
            vizier_link = ti["vizier_link"]
            in_ned = ti["in_ned"]
            object_types = ti["object_types"]
            version_number = ti["version_number"]
            last_updated = ti["last_updated"]
            legacy_table = ti["legacy_table"]
            old_table_name = ti["old_table_name"]
            weight = ti["object_type_accuracy"]

            number_of_rows = str(number_of_rows)
            thisLen = len(number_of_rows)
            newNumber = ""
            count = 0
            while count < thisLen:
                count += 1
                newNumber = number_of_rows[-count] + newNumber
                if count % 3 == 0:
                    newNumber = "," + newNumber
            if newNumber[0] == ",":
                newNumber = newNumber[1:]

            if len(vizier_link) and vizier_link != 0 and vizier_link != "0":
                vizier_link = u"[✓](%(vizier_link)s)" % locals()
            else:
                vizier_link = u""

            if in_ned:
                in_ned = u"✓"
            else:
                in_ned = u""

            rows += u"""
| <sub>%(table_name)s</sub> | <sub>[%(description)s](%(url)s)</sub> | <sub>[%(reference_text)s](%(reference_url)s)</sub> | <sub>%(newNumber)s</sub> | <sub>%(vizier_link)s</sub> | <sub>%(in_ned)s</sub> | <sub>%(object_types)s</sub> | <sub>%(weight)s</sub> |""" % locals()

        self.mdTable = header + rows

        header = u"""
| <sub>View Name</sub> | <sub>Number Rows</sub> | <sub>Object Type</sub> |
| :--- | :--- | :--- |"""

        rows = u""
        for ti in self.viewInfo:
            view_name = ti["view_name"]
            number_of_rows = ti["number_of_rows"]
            object_type = ti["object_type"]

            number_of_rows = str(number_of_rows)
            thisLen = len(number_of_rows)
            newNumber = ""
            count = 0
            while count < thisLen:
                count += 1
                newNumber = number_of_rows[-count] + newNumber
                if count % 3 == 0:
                    newNumber = "," + newNumber
            if newNumber[0] == ",":
                newNumber = newNumber[1:]

            rows += u"""
| <sub>%(view_name)s</sub> | <sub>%(newNumber)s</sub> | <sub>%(object_type)s</sub> |""" % locals()

        self.mdView = header + rows

        header = u"""
| <sub>Table Name</sub> | <sub>Description</sub> | <sub>Reference</sub> | <sub>Number Rows</sub> | <sub>Objects</sub> |
| :--- | :--- | :--- | :--- | :--- | :--- | :--- | :--- |"""

        rows = u""
        for ti in self.streamInfo:
            table_name = ti["table_name"]
            description = ti["description"]
            url = ti["url"]
            number_of_rows = ti["number_of_rows"]
            reference_url = ti["reference_url"]
            reference_text = ti["reference_text"]
            notes = ti["notes"]
            vizier_link = ti["vizier_link"]
            in_ned = ti["in_ned"]
            object_types = ti["object_types"]
            version_number = ti["version_number"]
            last_updated = ti["last_updated"]
            legacy_table = ti["legacy_table"]
            old_table_name = ti["old_table_name"]

            number_of_rows = str(number_of_rows)
            thisLen = len(number_of_rows)
            newNumber = ""
            count = 0
            while count < thisLen:
                count += 1
                newNumber = number_of_rows[-count] + newNumber
                if count % 3 == 0:
                    newNumber = "," + newNumber
            if newNumber[0] == ",":
                newNumber = newNumber[1:]

            if len(vizier_link) and vizier_link != 0 and vizier_link != "0":
                vizier_link = u"[✓](%(vizier_link)s)" % locals()
            else:
                vizier_link = u""

            if in_ned:
                in_ned = u"✓"
            else:
                in_ned = u""

            rows += u"""
| <sub>%(table_name)s</sub> | <sub>[%(description)s](%(url)s)</sub> | <sub>[%(reference_text)s](%(reference_url)s)</sub> | <sub>%(newNumber)s</sub> | <sub>%(object_types)s</sub> |""" % locals()

        self.mdStream = header + rows

        self.log.info('completed the ``_create_md_tables`` method')
        return None

    # use the tab-trigger below for new method
    def _write_wiki_pages(
            self):
        """
        *write wiki pages*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _write_wiki_pages method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_write_wiki_pages`` method')

        import codecs
        from datetime import datetime, date, time
        pathToWriteFile = self.settings[
            "sherlock wiki root"] + "/Crossmatch-Catalogue Tables.md"
        writeFile = codecs.open(pathToWriteFile, encoding='utf-8', mode='w')
        now = datetime.now()
        now = now.strftime("%Y-%m-%d %H:%M")
        lastUpdated = """Last Updated %(now)s  
""" % locals()

        writeFile.write(lastUpdated + self.mdTable)
        writeFile.close()

        pathToWriteFile = self.settings[
            "sherlock wiki root"] + "/Crossmatch-Catalogue Views.md"
        writeFile = codecs.open(pathToWriteFile, encoding='utf-8', mode='w')
        now = datetime.now()
        now = now.strftime("%Y-%m-%d %H:%M")
        lastUpdated = """Last Updated %(now)s  
""" % locals()

        writeFile.write(lastUpdated + self.mdView)
        writeFile.close()

        pathToWriteFile = self.settings[
            "sherlock wiki root"] + "/Crossmatch-Catalogue Streams.md"
        writeFile = codecs.open(pathToWriteFile, encoding='utf-8', mode='w')
        now = datetime.now()
        now = now.strftime("%Y-%m-%d %H:%M")
        lastUpdated = """Last Updated %(now)s  
""" % locals()

        writeFile.write(lastUpdated + self.mdStream)
        writeFile.close()

        self.log.info('completed the ``_write_wiki_pages`` method')
        return None

    # use the tab-trigger below for new method
    def _update_github(
            self):
        """
        *update github*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _update_github method
            - @review: when complete add logging
        """
        self.log.info('starting the ``_update_github`` method')

        from subprocess import Popen, PIPE, STDOUT
        gdir = self.settings["sherlock wiki root"]
        cmd = """cd %(gdir)s && git pull origin master && git add --all && git commit -m "x" && git push origin master""" % locals()
        p = Popen(cmd, stdout=PIPE, stdin=PIPE, shell=True)
        output = p.communicate()[0]
        print output
        self.log.debug('output: %(output)s' % locals())

        self.log.info('completed the ``_update_github`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx

# xt-class-tmpx


###################################################################
# PUBLIC FUNCTIONS                                                #
###################################################################
# xt-worker-def

# use the tab-trigger below for new function
# xt-def-with-logger

###################################################################
# PRIVATE (HELPER) FUNCTIONS                                      #
###################################################################

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
# SHERLOCK #
: INFERING TRANSIENT-SOURCE CLASSIFICATIONS FROM SPATIALLY CROSS-MATCHED CATALOGUED SOURCES :
=============================================================================================

Usage:
    sherlock match [--update] [-s <pathToSettingsFile>]
    sherlock clean [-s <pathToSettingsFile>]
    sherlock wiki [-s <pathToSettingsFile>]

    -h, --help            show this help message
    -v, --version         show version
    -s, --settings        the settings file
    -l, --transientlistId          the id of the transient list to classify
    -u, --update          update the transient database with new classifications and crossmatches
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from dryxPython import logs as dl
from dryxPython import commonutils as dcu
from fundamentals import tools, times
from classifier import classifier
from cleanup_database_tables import cleanup_database_tables
from commonutils import update_wiki_pages
# from ..__init__ import *


def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]


def main(arguments=None):
    """
    *The main function used when ``cl_utils.py`` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="DEBUG",
        options_first=False,
        projectName="sherlock"
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # unpack remaining cl arguments using `exec` to setup the variable names
    # automatically
    for arg, val in arguments.iteritems():
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        if isinstance(val, str) or isinstance(val, unicode):
            exec(varname + " = '%s'" % (val,))
        else:
            exec(varname + " = %s" % (val,))
        if arg == "--dbConn":
            dbConn = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = dcu.get_now_sql_datetime()
    log.debug(
        '--- STARTING TO RUN THE cl_utils.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in locals() and interactiveFlag:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    # call the worker function
    # x-if-settings-or-database-credientials

    if match:
        sherlock = classifier(
            log=log,
            settings=settings,
            update=updateFlag
        )
        sherlock.get()
    if clean:
        cleaner = cleanup_database_tables(
            log=log,
            settings=settings
        )
        cleaner.get()
    if wiki:
        updateWiki = update_wiki_pages(
            log=log,
            settings=settings
        )
        updateWiki.get()

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = dcu.get_now_sql_datetime()
    runningTime = dcu.calculate_time_difference(startTime, endTime)
    log.debug('-- FINISHED ATTEMPT TO RUN THE cl_utils.py AT %s (RUNTIME: %s) --' %
              (endTime, runningTime, ))

    return


if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*The conesearch object for sherlock*

:Author:
    David Young

:Date Created:
    July 1, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import math
from docopt import docopt
from HMpTy import htm
from cStringIO import StringIO
import numpy as np
from dryxPython import logs as dl
from dryxPython import astrotools as dat
from dryxPython import mysql as dms
from dryxPython import commonutils as dcu
from fundamentals import tools, times


class conesearcher():

    """
    *The worker class for the conesearcher module*

    **Key Arguments:**
        - ``dbConn`` -- mysql database connection
        - ``log`` -- logger
        - ``ra`` -- ra of location to search
        - ``dec`` -- dec of location to search
        - ``tableName`` -- the name of the database table to perform conesearch on
        - ``radius`` -- radius of the conesearch to perform (arcsec)
        - ``colMaps`` -- maps of the important column names for each table/view in the crossmatch-catalogues database
        - ``settings`` -- the settings dictionary
        - ``queryType`` -- queryType ["quick" | "full" | "count"]
        - ``htmLevel`` -- htmLevel [16 | 21]
        - ``nearestOnly`` -- return only the nearest object if true
        - ``physicalSearch`` -- is this a physical search (False for angular search only)
        - ``transType`` -- type of transient if match is found
    """
    # Initialisation

    def __init__(
            self,
            log,
            ra,
            dec,
            tableName,
            radius,
            colMaps,
            dbConn=False,
            settings=False,
            htmLevel=16,
            queryType="quick",
            nearestOnly=False,
            physicalSearch=False,
            transType=False
    ):
        self.log = log
        log.debug("instansiating a new '_conesearcher' object")
        self.dbConn = dbConn
        self.settings = settings
        self.ra = ra
        self.dec = dec
        self.htmLevel = htmLevel
        self.queryType = queryType
        self.radius = radius
        self.tableName = tableName
        self.nearestOnly = nearestOnly
        self.colMaps = colMaps
        self.physicalSearch = physicalSearch
        self.transType = transType
        self.mesh16 = htm.HTM(16)

        # xt-self-arg-tmpx

        # VARIABLE DATA ATRRIBUTES
        self.message = ""

        # INITIAL ACTIONS
        # FLIP QUERYTYPE TO INT
        for l, n in zip(["quick", "full", "count"], [1, 2, 3]):
            if self.queryType == l:
                self.queryType = n

        # CHECK HTM LEVELS
        if htmLevel not in (16, 20):
            self.log.error('Must be HTM level 16 or 20' % locals())
            return "Must be HTM level 16 or 20", []

        # CONVERT RA AND DEC TO DEGREES
        try:
            self.ra = dat.ra_sexegesimal_to_decimal.ra_sexegesimal_to_decimal(
                self.ra)
        except:
            pass
        try:
            self.dec = dat.declination_sexegesimal_to_decimal.declination_sexegesimal_to_decimal(
                self.dec)
        except:
            pass

        return None

    # METHOD ATTRIBUTES
    def get(self):
        """
        *get the conesearcher object*

        **Return:**
            - ``message`` -- message of success/failure
            - ``results`` -- the results for the conesearch
        """
        self.log.debug('starting the ``get`` method')

        self._build_sql_query_from_htm()

        # RETURN RESULTS IN BATCHES TO AVOID MEMORY ISSUES
        returnLimit = 700000
        offset = 0
        resultLen = returnLimit
        results = []
        while resultLen == returnLimit:
            resultSet, resultLen = self._grab_conesearch_results_from_db(
                returnLimit=returnLimit,
                offset=offset
            )
            results += resultSet
            offset += returnLimit
            returnLen = len(resultSet)
            # print """    %(returnLen)s/%(resultLen)s results matched""" %
            # locals()

        # SORT BY SEPARATION
        from operator import itemgetter
        self.results = sorted(results, key=itemgetter(0))

        self.log.debug('completed the ``get`` method')
        return self.message, self.results

    def _build_sql_query_from_htm(
            self):
        """
        *build sql query from htm*
        """
        self.log.debug('starting the ``_build_sql_query_from_htm`` method')

        # BUILD WHERE SECTION OF CLAUSE
        self.radius = float(self.radius)

        # CREATE AN ARRAY OF RELEVANT HTMIDS AND FIND MAX AND MIN
        thisArray = self.mesh16.intersect(
            self.ra, self.dec, self.radius / 3600., inclusive=True)
        hmax = thisArray.max()
        hmin = thisArray.min()

        # # IF JUMP BETWEEN THIS HTMINDEX AND NEXT IS > 200 THEN CREATE A NEW
        # # ARRAY (FOR BETWEEN STATEMENTS)
        # theseBetweens = np.split(thisArray, np.where(
        #     np.diff(thisArray) > 1)[0] + 1)

        # htmWhereClause = []
        # singleIds = []
        # for bet in theseBetweens:
        #     bMin, bMax = bet.min(), bet.max()
        #     if bMin == bMax:
        #         singleIds.append(str(bMin))
        #     else:
        #         htmWhereClause.append(
        #             "(htm16ID between %(bMin)s and %(bMax)s)" % locals())

        # if len(htmWhereClause):
        #     htmWhereClause = " OR ".join(htmWhereClause)
        # else:
        #     htmWhereClause = ""
        # singleIds = ",".join(singleIds)
        # if len(singleIds):
        #     singleIds = "htm16ID in (%(singleIds)s)" % locals()
        # else:
        #     singleIds = ""

        # if len(singleIds) and len(htmWhereClause):
        #     htmWhereClause = htmWhereClause + " or "

        # htmWhereClause = "where %(htmWhereClause)s %(singleIds)s" % locals(
        # )

        ratio = float(hmax - hmin + 1) / float(thisArray.size)
        if ratio < 100 or thisArray.size > 2000:
            htmWhereClause = "where htm16ID between %(hmin)s and %(hmax)s" % locals(
            )
        else:
            s = StringIO()
            np.savetxt(s, thisArray, fmt='%d', newline=",")
            thesHtmIds = s.getvalue()[:-1]
            htmWhereClause = "where htm16ID in (%(thesHtmIds)s)" % locals()

        # DECIDE WHAT COLUMNS TO REQUEST
        if self.queryType == 1:
            columns = self.quickColumns
        elif self.queryType == 3:
            columns = ['count(*) number']
        else:
            columns = []
            for k, v in self.colMaps[self.tableName].iteritems():
                if "colname" in k.lower() and v:
                    if "filterName" in k:
                        if "col_" in v:
                            v = v.replace("col_", "")
                            columns.append(v)
                        else:
                            columns.append("'%(v)s' as '%(k)s'" % locals())
                    else:
                        columns.append(v)

        columns = ','.join(columns)
        tableName = self.tableName

        # FINALLY BUILD THE FULL QUERY
        self.sqlQuery = "select %(columns)s from %(tableName)s %(htmWhereClause)s" % locals(
        )

        self.log.debug('completed the ``_build_sql_query_from_htm`` method')
        return None

    def _grab_conesearch_results_from_db(
            self,
            returnLimit=None,
            offset=None):
        """*grab conesearch results from db*
        """

        self.log.debug(
            'starting the ``_grab_conesearch_results_from_db`` method')

        # ACCOUNT FOR TYPE OF SEARCH
        if self.physicalSearch == False and self.transType == "SN":
            where = ""
            if self.colMaps[self.tableName]["redshiftColName"]:
                where += " and %s is null" % (
                    self.colMaps[self.tableName]["redshiftColName"],)
            if self.colMaps[self.tableName]["distanceColName"]:
                where += " and %s is null" % (
                    self.colMaps[self.tableName]["distanceColName"],)
            if self.colMaps[self.tableName]["semiMajorColName"]:
                where += " and %s is null" % (
                    self.colMaps[self.tableName]["semiMajorColName"],)
            self.sqlQuery += where
        elif self.physicalSearch == True:
            where = ""
            if self.colMaps[self.tableName]["redshiftColName"]:
                where += " or %s is not null" % (
                    self.colMaps[self.tableName]["redshiftColName"],)
            if self.colMaps[self.tableName]["distanceColName"]:
                where += " or %s is not null" % (
                    self.colMaps[self.tableName]["distanceColName"],)
            if self.colMaps[self.tableName]["semiMajorColName"]:
                where += " or %s is not null" % (
                    self.colMaps[self.tableName]["semiMajorColName"],)
            if len(where):
                where = " and (" + where[4:] + ")"
                self.sqlQuery += where

        # SETUP OFFSETS
        if returnLimit != None and offset != None:
            sqlQueryExtra = " limit %(returnLimit)s offset %(offset)s" % locals(
            )
        else:
            sqlQueryExtra = ""

        results = []
        # print "START DB"
        rows = dms.execute_mysql_read_query(
            sqlQuery=self.sqlQuery + sqlQueryExtra,
            dbConn=self.dbConn,
            log=self.log
        )
        # print "END DB"

        resultLen = len(rows)
        # print resultLen

        if len(rows):
            # IF ONLY A COUNT(*)
            if self.queryType == 3:
                results = [[0.0, rows[0]['number']]]
                return "Count", results

            # CALCULATE THE ANGULAR SEPARATION FOR EACH ROW
            raList = []
            decList = []
            for row in rows:
                if "guide_star" in self.tableName:
                    # Guide star cat RA and DEC are in RADIANS
                    ra2 = math.degrees(
                        row[self.colMaps[self.tableName]["raColName"]])
                    dec2 = math.degrees(
                        row[self.colMaps[self.tableName]["decColName"]])
                else:
                    ra2 = row[self.colMaps[self.tableName]["raColName"]]
                    dec2 = row[self.colMaps[self.tableName]["decColName"]]
                raList.append(ra2)
                decList.append(dec2)

            # CREATE TWO ARRAYS OF RA,DEC (1. TRANSIENT & 2. DATABASE RETURNS)
            tRa = np.array([self.ra])
            tDec = np.array([self.dec])
            raList = np.array(raList)
            decList = np.array(decList)
            indexList1, indexList2, separation = self.mesh16.match(
                tRa, tDec, raList, decList, self.radius / 3600., maxmatch=0)
            for i in range(indexList1.size):
                results.append([separation[i] * 3600., rows[indexList2[i]]])
            # for i in xrange(raList.size):
            #     results.append([0.1 * 3600., rows[i]])

            # IF NEAREST ONLY REQUESTED
            if self.nearestOnly == True:
                results = [results[0]]
        else:
            tableName = self.tableName
            self.message = "No matches from %(tableName)s." % locals()

        self.log.debug(
            'completed the ``_grab_conesearch_results_from_db`` method')
        return results, resultLen

    # use the tab-trigger below for new method
    # xt-class-method



import os
import nose
import shutil
import yaml
from sherlock import classifier
from sherlock.utKit import utKit


# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()

# load settings
stream = file(pathToInputDir + "/sherlock.yaml", 'r')
settings = yaml.load(stream)
stream.close()


# xnose-class-to-test-main-command-line-function-of-module

# @action: reset object_classifications to 0 for testing
# update tcs_transient_objects set object_classification = 0 where
# detection_list_id = 2;

class test_classifier():

    def test_classifier_function(self):
        kwargs = {}
        kwargs["log"] = log
        kwargs["settings"] = settings
        kwargs["transientIdList"] = [
            1000006110041705700, 1001901011281636100, 1000519791325919200]
        kwargs["update"] = True
        # xt-kwarg_key_and_value
        testObject = classifier(**kwargs)
        classifications = testObject.get()
        print classifications
        testObject.close()

    # def test_classifier_function2(self):
    #     kwargs = {}
    #     kwargs["log"] = log
    #     kwargs["settings"] = settings
    #     # xt-kwarg_key_and_value
    #     testObject = classifier(**kwargs)
    #     testObject.get()
    #     testObject.close()

    # def test_classifier_function2(self):
    #     kwargs = {}
    #     kwargs["log"] = log
    #     kwargs["settings"] = settings
    #     kwargs["workflowListId"] = "good"
    #     # xt-kwarg_key_and_value
    #     testObject = classifier(**kwargs)
    #     testObject.get()
    #     testObject.close()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



from milliquas import milliquas
from veron import veron
from tmp_sdss import tmp_sdss
from pessto_marshall_streams import pessto_marshall_streams
from ifs_galaxies import ifs_galaxies
import importers_clutils
from _base_importer import _base_importer

import ned_d
import ned_conesearch



from setuptools import setup, find_packages
import os

moduleDirectory = os.path.dirname(os.path.realpath(__file__))
exec(open(moduleDirectory + "/sherlock/__version__.py").read())


def readme():
    with open(moduleDirectory + '/README.rst') as f:
        return f.read()

install_requires = [
    'pyyaml',
    'qub-sherlock',
    'fundamentals',
    'astrocalc'
]

# READ THE DOCS SERVERS
exists = os.path.exists("/home/docs/")
if exists:
    c_exclude_list = ['healpy', 'astropy',
                      'numpy', 'wcsaxes', 'HMpTy', 'ligo-gracedb']
    for e in c_exclude_list:
        try:
            install_requires.remove(e)
        except:
            pass

setup(name="qub-sherlock",
      version=__version__,
      description="A contextual classifier for astronomical transient sources. Sherlock mines a library of historical and on-going survey data to search for the source of a transient event, and predicting the classification of the event based on that associated data.",
      long_description=readme(),
      classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Topic :: Utilities',
      ],
      keywords=['tools, classification, science'],
      url='https://github.com/thespacedoctor/sherlock',
      download_url='https://github.com/thespacedoctor/sherlock/archive/v%(__version__)s.zip' % locals(
      ),
      author='David Young',
      author_email='davidrobertyoung@gmail.com',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      install_requires=install_requires,
      test_suite='nose.collector',
      tests_require=['nose', 'nose-cover3'],
      entry_points={
          'console_scripts': [
              'sherlock=sherlock.cl_utils:main',
              'sherlock-importers=sherlock.imports.importers_clutils:main'
          ],
      },
      zip_safe=False)
